searchNodes=[{"doc":"Functions to make working with prefixes easier, especially IP prefixes (IPv4 and IPv6). Pfx defines a prefix as a struct with a number of bits and a maximum maxlen length. Hence a Pfx struct represents some domain-specific value, like an IPv4/6 address or network, a MAC address, a MAC OUI range or something completely different. A Pfx struct can be created with Pfx.new/2 using: a bitstring/0 and a non_neg_integer/0 for the maximum length, a Pfx.t/0 and a non_neg_integer/0 for a (new) maximum length. This allows for the creation of any sort of prefix. Use Pfx.new/1 to create a prefix struct from: a Pfx.ip_address/0 , a Pfx.ip_prefix/0 , or a binary/0 , a string in IPv4 CIDR, IPv6, EUI-48 or EUI-64 format This creates an IPv4, IPv6, EUI-48 or an EUI-64 prefix. Other means to create prefixes include Pfx.from_mac/1 and Pfx.from_hex/1 . Several functions, like Pfx.unique_local?/1 are more IP oriented, and are included along with the more generic Pfx functions (like Pfx.cut/3 ) in order to have one module to rule them all. Functions generally accept either a Pfx.t/0 , a Pfx.ip_address/0 , a Pfx.ip_prefix/0 or a binary and yield their result in the same fashion: iex&gt; hosts ( &quot;10.10.10.0/30&quot; ) [ &quot;10.10.10.0&quot; , &quot;10.10.10.1&quot; , &quot;10.10.10.2&quot; , &quot;10.10.10.3&quot; ] iex&gt; hosts ( { { 10 , 10 , 10 , 0 } , 30 } ) [ { { 10 , 10 , 10 , 0 } , 32 } , { { 10 , 10 , 10 , 1 } , 32 } , { { 10 , 10 , 10 , 2 } , 32 } , { { 10 , 10 , 10 , 3 } , 32 } ] iex&gt; hosts ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 :: 6 &gt;&gt; , maxlen : 32 } ) [ % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 3 &gt;&gt; , maxlen : 32 } , ] # adopt representation of first argument iex&gt; band ( { 10 , 10 , 10 , 1 } , &quot;255.255.255.0&quot; ) { 10 , 10 , 10 , 0 } iex&gt; multicast? ( &quot;ff00::1&quot; ) true # MAC OUI prefix iex&gt; keep ( &quot;aa:bb:cc:dd:ee:ff&quot; , 24 ) &quot;AA-BB-CC-00-00-00/24&quot; # get IPv4 from a IPv4 compatible IPv6 address iex&gt; cut ( &quot;::1.2.3.4&quot; , - 1 , - 32 ) &quot;1.2.3.4&quot; # partition a range of address space iex&gt; partition_range ( &quot;10.10.10.0&quot; , &quot;10.10.10.33&quot; ) [ &quot;10.10.10.0/27&quot; , &quot;10.10.10.32/31&quot; ] Validity The Pfx.new/2 function will silently clip the provided bits -string to maxlen -bits when needed, since a Pfx struct named pfx is valid, iff: bit_size(pfx.bits) &lt;= pfx.maxlen , and where pfx.maxlen is a non_neg_integer/0 Keep that in mind when instantiating directly or updating a Pfx , otherwise functions will choke on it. Same goes for Pfx.ip_address/0 representations, which must be a valid :inet.ip_address() , representing either an IPv4 or IPv6 address through a tuple of four 8 -bit wide numbers or eight 16 -bit wide numbers. If used as the first element in a Pfx.ip_prefix/0 tuple, the second element is interpreted as the mask, used to clip the bitstring when creating the Pfx struct. For example: {{1, 1, 1, 0}, 24} is the same as 1.1.1.0/24 . IPv4 masks must be in range 0..32 and IPv6 masks in range 0..128 . The resulting Pfx will have its maxlen set to 32 for IPv4 tuples and 128 for IPv6 tuples. Last but not least, binaries are interpreted as either an IPv4 in CIDR-notation, an IPv6 address/prefix, an EUI-48 or EUI-64 formatted string. # IPv4 iex&gt; new ( &quot;1.2.3.4&quot; ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } iex&gt; new ( { 1 , 2 , 3 , 4 } ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } iex&gt; new ( &quot;1.2.3.0/24&quot; ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } iex&gt; new ( { { 1 , 2 , 3 , 0 } , 24 } ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } # IPv6 iex&gt; new ( &quot;acdc:1975::&quot; ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1975 :: 16 , 0 :: 96 &gt;&gt; , maxlen : 128 } iex&gt; new ( { 44252 , 6517 , 0 , 0 , 0 , 0 , 0 , 0 } ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1975 :: 16 , 0 :: 96 &gt;&gt; , maxlen : 128 } # EUI-48 iex&gt; new ( &quot;00-88-88-88-88-88&quot; ) % Pfx { bits : &lt;&lt; 0 , 0x88 , 0x88 , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 48 } iex&gt; new ( &quot;0088.8888.8888&quot; ) % Pfx { bits : &lt;&lt; 0 , 0x88 , 0x88 , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 48 } # EUI-64 iex&gt; new ( &quot;02-88-88-FF-FE-88-88-88&quot; ) % Pfx { bits : &lt;&lt; 0x02 , 0x88 , 0x88 , 0xFF , 0xFE , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; new ( &quot;0288.88FF.FE88.8888&quot; ) % Pfx { bits : &lt;&lt; 0x02 , 0x88 , 0x88 , 0xFF , 0xFE , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 64 } Ancient tradition Pfx.new/1 accepts CIDR-strings which are ultimately processed using erlang's :inet.parse_address which, at the time of writing, still honors the ancient linux tradition of injecting zero's (rather than appending them) when presented with less than four IPv4 digits in a CIDR string. # &quot;d&quot; -&gt; &quot;0.0.0.d&quot; iex&gt; new ( &quot;10&quot; ) |&gt; format ( ) &quot;0.0.0.10&quot; iex&gt; new ( &quot;10/8&quot; ) |&gt; format ( ) &quot;0.0.0.0/8&quot; # &quot;d1.d2&quot; -&gt; &quot;d1.0.0.d2&quot; iex&gt; new ( &quot;10.10&quot; ) |&gt; format ( ) &quot;10.0.0.10&quot; iex&gt; new ( &quot;10.10/16&quot; ) |&gt; format ( ) &quot;10.0.0.0/16&quot; # &quot;d1.d2.d3&quot; -&gt; &quot;d1.d2.0.d3&quot; iex&gt; new ( &quot;10.10.10&quot; ) |&gt; format ( ) &quot;10.10.0.10&quot; iex&gt; new ( &quot;10.10.10/24&quot; ) |&gt; format ( ) &quot;10.10.0.0/24&quot; Bottom line: never go short, you may be unpleasantly surprised. EUI-64's Since a string is first parsed as an IP prefix, EUI-64's like &quot;11:22:33:44:55:66:77:88&quot; will come out as an IPv6 prefix with their maxlen property set to 128 . So, when parsing EUI's that might use ':'-s as punctuation, use Pfx.from_mac/1 , which also supports the tuple formats. Like Pfx.new/1 , this function always returns a Pfx.t/0 -struct. # new/1 parses EUI-64&#39;s like these correctly: iex&gt; new ( &quot;1122.3344.5566.7788&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; new ( &quot;11-22-33-44-55-66-77-88&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } # but new/1 turns this valid EUI-64 into IPv6 due to &#39;:&#39;-punctuation used: iex&gt; new ( &quot;01:02:03:04:05:06:07:08&quot; ) % Pfx { bits : &lt;&lt; 0x1 :: 16 , 0x2 :: 16 , 0x3 :: 16 , 0x4 :: 16 , 0x5 :: 16 , 0x6 :: 16 , 0x7 :: 16 , 0x8 :: 16 &gt;&gt; , maxlen : 128 } # in this case, use from_mac/1 iex&gt; from_mac ( &quot;01:02:03:04:05:06:07:08&quot; ) % Pfx { bits : &lt;&lt; 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 &gt;&gt; , maxlen : 64 } # and supports digit-styled EUI&#39;s iex&gt; from_mac ( { 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 } ) % Pfx { bits : &lt;&lt; 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 &gt;&gt; , maxlen : 64 } # from {{digits}, len}, keeping first 3 bytes iex&gt; from_mac ( { { 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 } , 24 } ) % Pfx { bits : &lt;&lt; 0x1 , 0x2 , 0x3 &gt;&gt; , maxlen : 64 } Enumeration A Pfx.t/0 implements the Enumerable protocol: iex&gt; for ip &lt;- % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 0 :: 6 &gt;&gt; , maxlen : 32 } , do : ip [ % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 0 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 3 &gt;&gt; , maxlen : 32 } , ] String.Chars Pfx.t/0 implements the String.Chars protocol with some defaults for prefixes that formats prefixes with: maxlen: 32 as an IPv4 CIDR string, maxlen: 48 as a EUI-48 address string and maxlen: 64 as a EUI-64 address string maxlen: 128 as an IPv6 string Other maxlen 's will simply come out as a series of 8-bit numbers joined by &quot;.&quot; followed by /num_of_bits . The latter is omitted if equal to pfx.bits length. If other formatting is required, use the Pfx.format/2 function, which takes some options that help shape the string representation for a Pfx struct. iex&gt; &quot; \#{ % Pfx { bits : &lt;&lt; 10 , 11 , 12 &gt;&gt; , maxlen : 32 } } &quot; &quot;10.11.12.0/24&quot; iex&gt; &quot; \#{ new ( &lt;&lt; 44252 :: 16 , 6518 :: 16 &gt;&gt; , 128 ) } &quot; &quot;acdc:1976:0:0:0:0:0:0/32&quot; iex&gt; &quot; \#{ % Pfx { bits : &lt;&lt; 0xA1 , 0xB2 , 0xC3 , 0xD4 , 0xE5 , 0xF6 &gt;&gt; , maxlen : 48 } } &quot; &quot;A1-B2-C3-D4-E5-F6&quot; iex&gt; &quot; \#{ new ( &lt;&lt; 1 , 2 , 3 , 4 , 5 &gt;&gt; , 64 ) } &quot; &quot;01-02-03-04-05-00-00-00/40&quot; # the enumeration example earlier, could also read: iex&gt; for ip &lt;- new ( &quot;1.2.3.0/30&quot; ) , do : &quot; \#{ ip } &quot; [ &quot;1.2.3.0&quot; , &quot;1.2.3.1&quot; , &quot;1.2.3.2&quot; , &quot;1.2.3.3&quot; ] # or iex&gt; for ip &lt;- new ( &quot;1.2.3.0/30&quot; ) , do : digits ( ip , 8 ) |&gt; elem ( 0 ) [ { 1 , 2 , 3 , 0 } , { 1 , 2 , 3 , 1 } , { 1 , 2 , 3 , 2 } , { 1 , 2 , 3 , 3 } ] Limitations A lot of Pfx -functions convert the Pfx.bits bitstring to an integer using Pfx.cast/1 , before performing some, often Bitwise -related, calculation on them. Luckily Elixir can handle pretty large numbers which seem mostly limited by the available system memory. Other functions, like Pfx.digits/2 return a tuple with numbers and are so limited by the maximum number of elements in a tuple (~16M+). So if you're taking this somewhere far, far away, heed these limitations before take off. Also, everything is done in Elixir with no extra, external dependencies. Usually fast enough, but if you really feel the need for speed, you might want to look elsewhere. Anyway, enough downplay, here are some more examples. Examples # IANA&#39;s OUI range 00-00-5e-xx-xx-xx iex&gt; new ( &quot;00-00-5e-00-00-00/24&quot; ) % Pfx { bits : &lt;&lt; 0 , 0 , 94 &gt;&gt; , maxlen : 48 } # IANA&#39;s VRRP MAC address range 00-00-5e-00-01-{VRID} iex&gt; vrrp_mac_range = new ( &quot;00-00-5e-00-01-00/40&quot; ) % Pfx { bits : &lt;&lt; 0 , 0 , 94 , 0 , 1 &gt;&gt; , maxlen : 48 } iex&gt; iex&gt; vrrp_mac = new ( &quot;00-00-5e-00-01-0f&quot; ) % Pfx { bits : &lt;&lt; 0 , 0 , 94 , 0 , 1 , 15 &gt;&gt; , maxlen : 48 } iex&gt; iex&gt; member? ( vrrp_mac , vrrp_mac_range ) true iex&gt; cut ( vrrp_mac , - 1 , - 8 ) |&gt; cast ( ) 15 iex&gt; new ( &quot;10.10.10.0/24&quot; ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } iex&gt; mask ( &quot;10.10.10.0/25&quot; ) &quot;255.255.255.128&quot; iex&gt; inv_mask ( &quot;10.10.10.0/25&quot; ) &quot;0.0.0.127&quot; iex&gt; dns_ptr ( &quot;acdc:1975::b1ba:2021&quot; ) &quot;1.2.0.2.a.b.1.b.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.5.7.9.1.c.d.c.a.ip6.arpa&quot; iex&gt; teredo_decode ( &quot;2001:0000:4136:e378:8000:63bf:3fff:fdd2&quot; ) %{ server : &quot;65.54.227.120&quot; , client : &quot;192.0.2.45&quot; , port : 40000 , flags : { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } , prefix : &quot;2001:0000:4136:e378:8000:63bf:3fff:fdd2&quot; } iex&gt; eui64_encode ( &quot;0288.8888.8888&quot; ) &quot;00-88-88-FF-FE-88-88-88&quot; Some functions have IP related names (like Pfx.teredo_decode/1 , but most of the times functions have generic names, since they apply to all sorts of prefixes, e.g. iex&gt; partition ( &quot;10.10.10.0/24&quot; , 26 ) [ &quot;10.10.10.0/26&quot; , &quot;10.10.10.64/26&quot; , &quot;10.10.10.128/26&quot; , &quot;10.10.10.192/26&quot; ] iex&gt; new ( &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , 32 ) ...&gt; |&gt; format ( width : 1 , unit : 8 ) &quot;00000001.00000010.00000011.00000100&quot; iex&gt; from_hex ( &quot;123456789abcdef&quot; ) ...&gt; |&gt; keep ( 20 ) % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x5 :: 4 &gt;&gt; , maxlen : 60 } iex&gt; brot ( &quot;1.2.3.4&quot; , 8 ) &quot;4.1.2.3&quot;","ref":"Pfx.html","title":"Pfx","type":"module"},{"doc":"Returns the address portion of given prefix without applying the mask (if any). Note that this has no real effect on a Pfx.t/0 or an ip_address/0 since there is no mask to ignore. Since no mask is applied, this always returns a full prefix without any bits masked off. Raises ArgumentError if given an invalid prefix. Examples iex&gt; address ( &quot;1.2.3.4/16&quot; ) &quot;1.2.3.4&quot; iex&gt; address ( { { 1 , 2 , 3 , 4 } , 16 } ) { { 1 , 2 , 3 , 4 } , 32 } iex&gt; address ( { { 0xacdc , 0x1976 , 0 , 0 , 0 , 0 , 0 , 1 } , 64 } ) { { 0xacdc , 0x1976 , 0 , 0 , 0 , 0 , 0 , 1 } , 128 } iex&gt; pfx = &quot;1.2.3.4/24&quot; iex&gt; new ( pfx ) . bits &lt;&lt; 1 , 2 , 3 &gt;&gt; iex&gt; new ( address ( pfx ) ) . bits &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; # no real effect iex&gt; address ( &quot;1.2.3.4&quot; ) &quot;1.2.3.4&quot; iex&gt; address ( { 1 , 2 , 3 , 4 } ) { 1 , 2 , 3 , 4 } iex&gt; address ( { 0xacdc , 0x1976 , 0 , 0 , 0 , 0 , 0 , 1 } ) { 0xacdc , 0x1976 , 0 , 0 , 0 , 0 , 0 , 1 } iex&gt; new ( &quot;1.2.3.4/16&quot; ) |&gt; address ( ) % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#address/1","title":"Pfx.address/1","type":"function"},{"doc":"Returns a bitwise AND of pfx1 and pfx2 . Both prefixes must have the same maxlen . The resulting prefix will have the same number of bits as the first argument. Examples iex&gt; band ( &quot;10.10.10.10&quot; , &quot;255.255.0.0&quot; ) &quot;10.10.0.0&quot; iex&gt; band ( &quot;10.10.10.0/24&quot; , &quot;255.255.0.0&quot; ) &quot;10.10.0.0/24&quot; iex&gt; x = new ( &lt;&lt; 128 , 129 , 130 , 131 &gt;&gt; , 32 ) iex&gt; y = new ( &lt;&lt; 255 , 255 &gt;&gt; , 32 ) iex&gt; band ( x , y ) % Pfx { bits : &lt;&lt; 128 , 129 , 0 , 0 &gt;&gt; , maxlen : 32 } iex&gt; iex&gt; band ( y , x ) % Pfx { bits : &lt;&lt; 128 , 129 &gt;&gt; , maxlen : 32 } # results adopt the format of the first argument iex&gt; band ( &quot;1.2.3.4&quot; , { 255 , 255 , 0 , 0 } ) &quot;1.2.0.0&quot; iex&gt; band ( { 1 , 2 , 3 , 4 } , &quot;255.255.0.0&quot; ) { 1 , 2 , 0 , 0 } iex&gt; band ( { { 1 , 2 , 3 , 4 } , 24 } , { 255 , 255 , 0 , 0 } ) { { 1 , 2 , 0 , 0 } , 24 } # honoring the ancient tradition iex&gt; band ( &quot;1.2.3.4&quot; , &quot;255.255&quot; ) &quot;1.0.0.4&quot; iex&gt; band ( &quot;10.10.0.0/16&quot; , &quot;255.0.0.0/24&quot; ) &quot;10.0.0.0/16&quot;","ref":"Pfx.html#band/2","title":"Pfx.band/2","type":"function"},{"doc":"Returns the bit-value at given position in pfx . A bit position is a 0 -based index from the left with range 0..maxlen-1 . A negative bit position is taken relative to Pfx.maxlen . Bits that are masked (bit position in the range of bit_size(pfx.bits) .. pfx.maxlen - 1 ) always yield 0 . Examples iex&gt; bit ( &quot;1.2.0.0&quot; , 14 ) 1 # same bit iex&gt; bit ( &quot;1.2.0.0&quot; , - 18 ) 1 iex&gt; bit ( &quot;1.2.0.0/16&quot; , 14 ) 1 iex&gt; bit ( { 1 , 2 , 0 , 0 } , 14 ) 1 iex&gt; bit ( { { 1 , 2 , 0 , 0 } , 16 } , 14 ) 1 iex&gt; bit ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } , 14 ) 1 # &#39;masked&#39; bits are deemed to be `0` iex&gt; bit ( &quot;1.2.0.0/16&quot; , 24 ) 0 # errors out on invalid positions iex&gt; bit ( &quot;255.255.255.255&quot; , 33 ) ** (ArgumentError) invalid bit position: 33 iex&gt; bit ( &quot;10.10.0.0/16&quot; , - 33 ) ** (ArgumentError) invalid bit position: -33","ref":"Pfx.html#bit/2","title":"Pfx.bit/2","type":"function"},{"doc":"Returns the concatenation of 1 or more series of bits of the given pfx . Examples iex&gt; bits ( &quot;1.2.3.4&quot; , [ { 0 , 8 } , { - 1 , - 8 } ] ) &lt;&lt; 1 , 4 &gt;&gt; iex&gt; bits ( &quot;1.2.3.0/24&quot; , [ { 0 , 8 } , { - 1 , - 8 } ] ) &lt;&lt; 1 , 0 &gt;&gt; iex&gt; bits ( { 1 , 2 , 3 , 4 } , [ { 0 , 8 } , { - 1 , - 8 } ] ) &lt;&lt; 1 , 4 &gt;&gt; iex&gt; bits ( { { 1 , 2 , 3 , 0 } , 24 } , [ { 0 , 8 } , { - 1 , - 8 } ] ) &lt;&lt; 1 , 0 &gt;&gt; iex&gt; bits ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } , [ { 0 , 8 } , { - 1 , - 8 } ] ) &lt;&lt; 1 , 4 &gt;&gt;","ref":"Pfx.html#bits/2","title":"Pfx.bits/2","type":"function"},{"doc":"Returns length bits, starting at position for given pfx . Negative position 's are relative to the end of the pfx.bits bitstring, while negative length will collect bits going left instead of to the right. Note that the bit at given position is always included in the result regardless of direction. Finally, a length of 0 results in an empty bitstring. Examples # last two bytes iex&gt; bits ( &quot;128.0.128.1&quot; , 16 , 16 ) &lt;&lt; 128 , 1 &gt;&gt; iex&gt; bits ( { 128 , 0 , 128 , 1 } , 16 , 16 ) # same &lt;&lt; 128 , 1 &gt;&gt; iex&gt; bits ( { 128 , 0 , 128 , 1 } , 31 , - 16 ) # same &lt;&lt; 128 , 1 &gt;&gt; iex&gt; bits ( { { 128 , 0 , 128 , 1 } , 32 } , 31 , - 16 ) # same &lt;&lt; 128 , 1 &gt;&gt; # first byte iex&gt; bits ( % Pfx { bits : &lt;&lt; 128 , 0 , 0 , 1 &gt;&gt; , maxlen : 32 } , 0 , 8 ) &lt;&lt; 128 &gt;&gt; # same as iex&gt; bits ( % Pfx { bits : &lt;&lt; 128 , 0 , 0 , 1 &gt;&gt; , maxlen : 32 } , 7 , - 8 ) &lt;&lt; 128 &gt;&gt; # missing bits are filled in as `0` iex&gt; x = new ( &lt;&lt; 128 &gt;&gt; , 32 ) iex&gt; bits ( x , 0 , 32 ) &lt;&lt; 128 , 0 , 0 , 0 &gt;&gt; iex&gt; x = new ( &lt;&lt; 128 &gt;&gt; , 32 ) iex&gt; bits ( x , 0 , 16 ) &lt;&lt; 128 , 0 &gt;&gt; iex&gt; x = new ( &lt;&lt; 128 &gt;&gt; , 32 ) iex&gt; bits ( x , 15 , - 16 ) &lt;&lt; 128 , 0 &gt;&gt; # the last 5 bits iex&gt; x = new ( &lt;&lt; 255 &gt;&gt; , 32 ) iex&gt; bits ( x , 7 , - 5 ) &lt;&lt; 0b11111 :: size ( 5 ) &gt;&gt;","ref":"Pfx.html#bits/3","title":"Pfx.bits/3","type":"function"},{"doc":"Returns a bitwise NOT of the bits in pfx . Results are returned in the same representation as given pfx . Examples iex&gt; bnot ( &quot;255.255.0.0&quot; ) &quot;0.0.255.255&quot; iex&gt; bnot ( { 255 , 255 , 0 , 0 } ) { 0 , 0 , 255 , 255 } iex&gt; bnot ( { { 255 , 255 , 0 , 0 } , 32 } ) { { 0 , 0 , 255 , 255 } , 32 } iex&gt; new ( &lt;&lt; 255 , 255 , 0 , 0 &gt;&gt; , 32 ) |&gt; bnot ( ) % Pfx { bits : &lt;&lt; 0 , 0 , 255 , 255 &gt;&gt; , maxlen : 32 } iex&gt; bnot ( &quot;5323:e689::/32&quot; ) &quot;acdc:1976:0:0:0:0:0:0/32&quot;","ref":"Pfx.html#bnot/1","title":"Pfx.bnot/1","type":"function"},{"doc":"Returns a bitwise OR of pfx1 and pfx2 . Both prefixes must have the same maxlen . The result will have the same number of bits as its first argument. Examples iex&gt; bor ( &quot;1.2.3.4&quot; , &quot;0.0.255.0&quot; ) &quot;1.2.255.4&quot; iex&gt; bor ( { 1 , 2 , 3 , 4 } , &quot;0.0.255.0&quot; ) { 1 , 2 , 255 , 4 } iex&gt; bor ( { { 1 , 2 , 3 , 4 } , 16 } , { 0 , 255 , 255 , 0 } ) { { 1 , 255 , 0 , 0 } , 16 } # same sized `bits` iex&gt; x = new ( &lt;&lt; 10 , 11 , 12 , 13 &gt;&gt; , 32 ) iex&gt; y = new ( &lt;&lt; 0 , 0 , 255 , 255 &gt;&gt; , 32 ) iex&gt; bor ( x , y ) % Pfx { bits : &lt;&lt; 10 , 11 , 255 , 255 &gt;&gt; , maxlen : 32 } # same `maxlen` but differently sized `bits`: missing bits are considered to be `0` iex&gt; bor ( &quot;10.11.12.13&quot; , &quot;255.255.0.0/16&quot; ) &quot;255.255.12.13&quot; # result has same number of bits as the first prefix iex&gt; bor ( &quot;10.10.0.0/16&quot; , &quot;255.255.255.255&quot; ) &quot;255.255.0.0/16&quot;","ref":"Pfx.html#bor/2","title":"Pfx.bor/2","type":"function"},{"doc":"Returns the broadcast prefix (full address) for given pfx . Yields the same result as Pfx.last/1 , included for nostalgia. Examples iex&gt; broadcast ( &quot;10.10.0.0/16&quot; ) &quot;10.10.255.255&quot; # a full address is its own broadcast address iex&gt; broadcast ( { 10 , 10 , 10 , 1 } ) { 10 , 10 , 10 , 1 } iex&gt; broadcast ( { { 10 , 10 , 10 , 1 } , 30 } ) { { 10 , 10 , 10 , 3 } , 32 } iex&gt; broadcast ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 255 &gt;&gt; , maxlen : 32 } iex&gt; broadcast ( % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1976 :: 16 &gt;&gt; , maxlen : 128 } ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1976 :: 16 , - 1 :: 96 &gt;&gt; , maxlen : 128 } iex&gt; broadcast ( &quot;acdc:1976::/112&quot; ) &quot;acdc:1976:0:0:0:0:0:ffff&quot;","ref":"Pfx.html#broadcast/1","title":"Pfx.broadcast/1","type":"function"},{"doc":"Rotates the bits of pfx by n positions. Positive n rotates right, negative rotates left. The length of the resulting bits stays the same. Examples iex&gt; brot ( &quot;1.2.3.4&quot; , 8 ) &quot;4.1.2.3&quot; iex&gt; brot ( &quot;1.2.3.4&quot; , - 8 ) &quot;2.3.4.1&quot; iex&gt; brot ( { 1 , 2 , 3 , 4 } , 8 ) { 4 , 1 , 2 , 3 } iex&gt; brot ( { { 1 , 2 , 3 , 4 } , 32 } , - 8 ) { { 2 , 3 , 4 , 1 } , 32 } # note: the `bits` &lt;&lt;1, 2&gt;&gt; get rotated (!) iex&gt; brot ( &quot;1.2.0.0/16&quot; , 8 ) &quot;2.1.0.0/16&quot; iex&gt; brot ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } , 8 ) % Pfx { bits : &lt;&lt; 4 , 1 , 2 , 3 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#brot/2","title":"Pfx.brot/2","type":"function"},{"doc":"Sets all bits of pfx to either 0 or 1 . If bit is not provided, it defaults to 0 . Examples iex&gt; bset ( &quot;1.1.1.0/24&quot; ) &quot;0.0.0.0/24&quot; iex&gt; bset ( &quot;1.1.1.0/24&quot; , 1 ) &quot;255.255.255.0/24&quot; iex&gt; bset ( { { 1 , 1 , 1 , 0 } , 24 } , 1 ) { { 255 , 255 , 255 , 0 } , 24 } iex&gt; bset ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 0 , 0 , 0 &gt;&gt; , maxlen : 32 } iex&gt; bset ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , 1 ) % Pfx { bits : &lt;&lt; 255 , 255 , 255 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#bset/2","title":"Pfx.bset/2","type":"function"},{"doc":"Performs an arithmetic shift left of the bits in pfx by n positions. A positive n shifts to the left, negative n shifts to the right. The length of the bits stays the same. Examples iex&gt; bsl ( &quot;1.2.3.4&quot; , 1 ) &quot;2.4.6.8&quot; iex&gt; bsl ( &quot;1.2.0.0/16&quot; , 2 ) &quot;4.8.0.0/16&quot; iex&gt; bsl ( { 1 , 2 , 3 , 4 } , 2 ) { 4 , 8 , 12 , 16 } # note: the `bits` &lt;&lt;1, 2&gt;&gt; get shifted left 2 bits iex&gt; bsl ( { { 1 , 2 , 0 , 0 } , 16 } , 2 ) { { 4 , 8 , 0 , 0 } , 16 } iex&gt; bsl ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } , 2 ) % Pfx { bits : &lt;&lt; 4 , 8 &gt;&gt; , maxlen : 32 } iex&gt; bsl ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } , - 2 ) % Pfx { bits : &lt;&lt; 0 , 64 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#bsl/2","title":"Pfx.bsl/2","type":"function"},{"doc":"Performs an arithmetic shift right the bits in pfx by n positions. A negative n actually shifts to the left. The length of the bits stays the same. Examples iex&gt; bsr ( &quot;1.2.0.0/16&quot; , 2 ) &quot;0.64.0.0/16&quot; # no mask, so all 32 bits get shifted iex&gt; bsr ( { 1 , 2 , 0 , 0 } , 2 ) { 0 , 64 , 128 , 0 } iex&gt; bsr ( { { 1 , 2 , 0 , 0 } , 16 } , 2 ) { { 0 , 64 , 0 , 0 } , 16 } iex&gt; bsr ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } , 2 ) % Pfx { bits : &lt;&lt; 0 , 64 &gt;&gt; , maxlen : 32 } # now shift to the left iex&gt; bsr ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } , - 2 ) % Pfx { bits : &lt;&lt; 4 , 8 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#bsr/2","title":"Pfx.bsr/2","type":"function"},{"doc":"Returns a bitwise XOR of pfx1 and pfx2 . Both prefixes must have the same maxlen . The result has the same number of bits as the first argument. Examples iex&gt; bxor ( &quot;10.11.12.13&quot; , &quot;255.255.0.0&quot; ) &quot;245.244.12.13&quot; iex&gt; bxor ( { 10 , 11 , 12 , 13 } , { 255 , 255 , 0 , 0 } ) { 245 , 244 , 12 , 13 } # mix &#39;n match iex&gt; bxor ( { { 10 , 11 , 12 , 13 } , 32 } , &quot;255.255.0.0&quot; ) { { 245 , 244 , 12 , 13 } , 32 } iex&gt; x = new ( &lt;&lt; 10 , 11 , 12 , 13 &gt;&gt; , 32 ) iex&gt; y = new ( &lt;&lt; 255 , 255 &gt;&gt; , 32 ) iex&gt; bxor ( x , y ) % Pfx { bits : &lt;&lt; 245 , 244 , 12 , 13 &gt;&gt; , maxlen : 32 } iex&gt; bxor ( &quot;255.255.0.0/16&quot; , &quot;10.11.12.13&quot; ) &quot;245.244.0.0/16&quot;","ref":"Pfx.html#bxor/2","title":"Pfx.bxor/2","type":"function"},{"doc":"Casts a pfx to an integer. After right padding the given pfx , its bits are interpreted as a number of maxlen bits wide. Empty prefixes evaluate to 0 , since all 'missing' bits are taken to be zero (even if pfx.maxlen is 0 ). See cut/3 for how this capability might be useful. Examples iex&gt; cast ( &quot;255.255.0.0&quot; ) 4294901760 iex&gt; cast ( &quot;255.255.0.0/16&quot; ) 4294901760 iex&gt; cast ( { 255 , 255 , 0 , 0 } ) 4294901760 iex&gt; cast ( { { 255 , 255 , 0 , 0 } , 32 } ) 4294901760 iex&gt; cast ( % Pfx { bits : &lt;&lt; 255 , 255 &gt;&gt; , maxlen : 32 } ) 4294901760 iex&gt; new ( &lt;&lt; 4294901760 :: 32 &gt;&gt; , 32 ) % Pfx { bits : &lt;&lt; 255 , 255 , 0 , 0 &gt;&gt; , maxlen : 32 } # missing bits filled in as `0`s iex&gt; cast ( % Pfx { bits : &lt;&lt; 255 &gt;&gt; , maxlen : 16 } ) 65280 iex&gt; cast ( % Pfx { bits : &lt;&lt; - 1 :: 128 &gt;&gt; , maxlen : 128 } ) 340282366920938463463374607431768211455 iex&gt; cast ( % Pfx { bits : &lt;&lt; &gt;&gt; , maxlen : 8 } ) 0 # a bit weird, but: iex&gt; cast ( % Pfx { bits : &lt;&lt; &gt;&gt; , maxlen : 0 } ) 0","ref":"Pfx.html#cast/1","title":"Pfx.cast/1","type":"function"},{"doc":"Compares pfx1 to pfx2 for sorting purposes. The result is one of: :eq prefix1 is equal to prefix2 :lt prefix1 has more bits or lies to the left of prefix2 :gt prefix1 has less bits or lies to the right of prefix2 The prefixes must have the same maxlen and are first compared by size (i.e. a shorter prefix is considered larger ), and second on their bitstring value. Examples iex&gt; compare ( &quot;10.0.0.0/8&quot; , &quot;11.0.0.0/8&quot; ) :lt iex&gt; compare ( &quot;10.0.0.0/8&quot; , { { 11 , 0 , 0 , 0 } , 8 } ) :lt iex&gt; compare ( { 10 , 0 , 0 , 0 } , { { 11 , 0 , 0 , 0 } , 16 } ) :lt iex&gt; compare ( new ( &lt;&lt; 10 &gt;&gt; , 32 ) , new ( &lt;&lt; 11 &gt;&gt; , 32 ) ) :lt # sort on prefixes, first on bit_size than bits-values iex&gt; list = [ &quot;10.11.0.0/16&quot; , &quot;10.10.10.0/24&quot; , &quot;10.10.0.0/16&quot; ] iex&gt; Enum . sort ( list , Pfx ) [ &quot;10.10.10.0/24&quot; , &quot;10.10.0.0/16&quot; , &quot;10.11.0.0/16&quot; ] # whereas regular sort does: # iex&gt; Enum . sort ( list ) [ &quot;10.10.0.0/16&quot; , &quot;10.10.10.0/24&quot; , &quot;10.11.0.0/16&quot; ] iex&gt; list = [ new ( &lt;&lt; 10 , 11 &gt;&gt; , 32 ) , new ( &lt;&lt; 10 , 10 , 10 &gt;&gt; , 32 ) , new ( &lt;&lt; 10 , 10 &gt;&gt; , 32 ) ] iex&gt; Enum . sort ( list , Pfx ) [ % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 11 &gt;&gt; , maxlen : 32 } ] # not advisable, but mixed representations are possible as well iex&gt; l = [ &quot;10.11.0.0/16&quot; , { { 10 , 10 , 10 , 0 } , 24 } , % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } ] iex&gt; Enum . sort ( l , Pfx ) [ { { 10 , 10 , 10 , 0 } , 24 } , % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } , &quot;10.11.0.0/16&quot; , ] # note: all prefixes must have the same `maxlen` iex&gt; compare ( new ( &lt;&lt; 10 &gt;&gt; , 32 ) , new ( &lt;&lt; 10 &gt;&gt; , 128 ) ) ** (ArgumentError) prefixes have different maxlen&#39;s: {%Pfx{bits: &quot;\\n&quot;, maxlen: 32}, %Pfx{bits: &quot;\\n&quot;, maxlen: 128}}","ref":"Pfx.html#compare/2","title":"Pfx.compare/2","type":"function"},{"doc":"Contrasts pfx1 to pfx2 . Contrasting two prefixes yields one of: :equal pfx1 is equal to pfx2 :more pfx1 is a more specific version of pfx2 :less pfx1 is a less specific version of pfx2 :left pfx1 is left-adjacent to pfx2 :right pfx1 is right-adjacent to pfx2 :disjoint pfx1 has no match with pfx2 whatsoever. Examples iex&gt; contrast ( &quot;10.10.0.0/16&quot; , &quot;10.10.0.0/16&quot; ) :equal iex&gt; contrast ( &quot;10.10.10.0/24&quot; , &quot;10.10.0.0/16&quot; ) :more iex&gt; contrast ( &quot;10.0.0.0/8&quot; , &quot;10.255.255.0/24&quot; ) :less iex&gt; contrast ( &quot;1.2.3.0/24&quot; , &quot;1.2.4.0/24&quot; ) :left iex&gt; contrast ( &quot;1.2.3.4/30&quot; , &quot;1.2.3.0/30&quot; ) :right iex&gt; contrast ( &quot;10.10.0.0/16&quot; , &quot;9.0.0.0/8&quot; ) :disjoint iex&gt; contrast ( &quot;10.10.0.0/16&quot; , % Pfx { bits : &lt;&lt; 10 , 12 &gt;&gt; , maxlen : 32 } ) :disjoint","ref":"Pfx.html#contrast/2","title":"Pfx.contrast/2","type":"function"},{"doc":"Cuts out a series of bits and turns it into its own Pfx . Extracts the bits and returns a new Pfx.t/0 with bits set to the bits extracted and maxlen set to the length of the bits -string. Examples iex&gt; cut ( &quot;::ffff:192.0.2.128&quot; , - 1 , - 32 ) &quot;192.0.2.128&quot; iex&gt; teredo = new ( &quot;2001:0:4136:e378:8000:63bf:3fff:fdd2&quot; ) iex&gt; # client iex&gt; cut ( teredo , 96 , 32 ) |&gt; bnot ( ) |&gt; format ( ) &quot;192.0.2.45&quot; iex&gt; # udp port iex&gt; cut ( teredo , 80 , 16 ) |&gt; bnot ( ) |&gt; cast ( ) 40000 iex&gt; # teredo server iex&gt; cut ( teredo , 32 , 32 ) |&gt; format ( ) &quot;65.54.227.120&quot; iex&gt; # flags iex&gt; cut ( teredo , 64 , 16 ) |&gt; digits ( 1 ) |&gt; elem ( 0 ) { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } 'Masked' bits are considered to be zero. # extract 2nd and 3rd byte: iex&gt; % Pfx { bits : &lt;&lt; 255 , 255 &gt;&gt; , maxlen : 32 } |&gt; cut ( 8 , 16 ) % Pfx { bits : &lt;&lt; 255 , 0 &gt;&gt; , maxlen : 16 } Less useful, but cut will mirror the representation given: iex&gt; cut ( &quot;10.11.12.13&quot; , 8 , 16 ) &quot;11.12&quot; iex&gt; cut ( { 1 , 2 , 3 , 4 } , 16 , 16 ) { 3 , 4 } iex&gt; cut ( { { 1 , 2 , 0 , 0 } , 16 } , 8 , 16 ) { { 2 , 0 } , 16 } Extraction must stay within maxlen of given pfx . # cannot exceed boundaries though: iex&gt; % Pfx { bits : &lt;&lt; 255 , 255 &gt;&gt; , maxlen : 32 } |&gt; cut ( 8 , 32 ) ** (ArgumentError) invalid index range: {8, 32}","ref":"Pfx.html#cut/3","title":"Pfx.cut/3","type":"function"},{"doc":"Returns a {{digit, ..}, length} representation of given pfx . The pfx is padded to its maximum length using 0 's and the resulting bits are grouped into digits , each width -bits wide. The resulting length denotes the prefix' original bit_size. Note: works best if the prefix' maxlen is a multiple of the width used, otherwise maxlen cannot be inferred from this format by tuple_size(digits) * width (e.g. by Pfx.undigits/2 ) Examples iex&gt; digits ( &quot;10.11.12.0/24&quot; , 8 ) { { 10 , 11 , 12 , 0 } , 24 } # mask is applied first iex&gt; digits ( &quot;10.11.12.13/24&quot; , 8 ) { { 10 , 11 , 12 , 0 } , 24 } iex&gt; digits ( &quot;acdc:1976::/32&quot; , 16 ) { { 44252 , 6518 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 } iex&gt; digits ( { { 0xacdc , 0x1976 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 } , 16 ) { { 44252 , 6518 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 } iex&gt; digits ( % Pfx { bits : &lt;&lt; 10 , 11 , 12 &gt;&gt; , maxlen : 32 } , 8 ) { { 10 , 11 , 12 , 0 } , 24 } iex&gt; digits ( % Pfx { bits : &lt;&lt; 10 , 11 , 12 , 1 :: 1 &gt;&gt; , maxlen : 32 } , 8 ) { { 10 , 11 , 12 , 128 } , 25 } iex&gt; digits ( % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x56 , 0x78 &gt;&gt; , maxlen : 128 } , 4 ) { { 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 }","ref":"Pfx.html#digits/2","title":"Pfx.digits/2","type":"function"},{"doc":"Returns a reverse DNS name (pointer) for given pfx . The prefix will be padded right with 0 -bits to a multiple of 8 for IPv4 prefixes and to a multiple of 4 for IPv6 prefixes. Note that this might give unexpected results. So dns_ptr/1 works best if the prefix given is actually a multiple of 4 or 8. Examples iex&gt; dns_ptr ( &quot;10.10.0.0/16&quot; ) &quot;10.10.in-addr.arpa&quot; # &quot;1.2.3.0/23&quot; actually encodes as %Pfx{bits: &lt;&lt;1, 2, 1::size(7)&gt;&gt;, maxlen: 32} # and padding right with 0-bits to a /24 yields the 1.2.2.0/24 ... iex&gt; dns_ptr ( &quot;1.2.3.0/23&quot; ) &quot;2.2.1.in-addr.arpa&quot; iex&gt; dns_ptr ( &quot;acdc:1976::/32&quot; ) &quot;6.7.9.1.c.d.c.a.ip6.arpa&quot; iex&gt; dns_ptr ( &quot;acdc:1975::b1ba:2021&quot; ) &quot;1.2.0.2.a.b.1.b.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.5.7.9.1.c.d.c.a.ip6.arpa&quot;","ref":"Pfx.html#dns_ptr/1","title":"Pfx.dns_ptr/1","type":"function"},{"doc":"Drops count lsb bits from given pfx . If count exceeds the actual number of bits in pfx , simply drops all bits. Examples iex&gt; drop ( &quot;1.2.3.0/31&quot; , 1 ) &quot;1.2.3.0/30&quot; iex&gt; drop ( &quot;1.2.3.2/31&quot; , 1 ) &quot;1.2.3.0/30&quot; iex&gt; drop ( &quot;1.2.3.128/25&quot; , 1 ) &quot;1.2.3.0/24&quot; # drops all iex&gt; drop ( &quot;1.2.3.0/24&quot; , 512 ) &quot;0.0.0.0/0&quot; iex&gt; drop ( &quot;2001:0db8:85a3:0000:0000:8a2e:0370:7334&quot; , 64 ) &quot;2001:db8:85a3:0:0:0:0:0/64&quot; iex&gt; drop ( { 1 , 2 , 3 , 4 } , 8 ) { 1 , 2 , 3 , 0 } iex&gt; drop ( { { 1 , 2 , 3 , 4 } , 32 } , 16 ) { { 1 , 2 , 0 , 0 } , 16 } iex&gt; drop ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } , 16 ) % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#drop/2","title":"Pfx.drop/2","type":"function"},{"doc":"Decodes a modified EUI-64 back into the original EUI-48 address. This function flips the 7-th bit and removes 16-bits from the middle. Those 16-bits should be 0xFFFE , but this is not checked or enforced. Examples iex&gt; eui64_decode ( &quot;0088.88FE.FF88.8888&quot; ) &quot;02-88-88-88-88-88&quot; iex&gt; eui64_decode ( &quot;0288.88FF.FE88.8888&quot; ) &quot;00-88-88-88-88-88&quot; iex&gt; eui64_decode ( &quot;02-88-88-FF-FE-88-88-88&quot; ) &quot;00-88-88-88-88-88&quot; iex&gt; eui64_decode ( { 0x02 , 0x88 , 0x88 , 0xFF , 0xFE , 0x88 , 0x88 , 0x88 } ) { 0x00 , 0x88 , 0x88 , 0x88 , 0x88 , 0x88 } iex&gt; new ( &quot;2001:db8:1:2:020c:29ff:fe0c:47d5&quot; ) ...&gt; |&gt; cut ( - 1 , - 64 ) ...&gt; |&gt; eui64_decode ( ) ...&gt; |&gt; format ( ) &quot;00-0C-29-0C-47-D5&quot;","ref":"Pfx.html#eui64_decode/1","title":"Pfx.eui64_decode/1","type":"function"},{"doc":"Creates a modified EUI-64 out of eui (an EUI-48 address or an EUI-64). This flips the 7-th bit (U/L - universal/local) and inserts 0xFFFE in the middle. The function assumes either an EUI-48 or EUI-64 address. In the latter case, it'll only flip the 7-th bit. Examples iex&gt; eui64_encode ( &quot;0088.8888.8888&quot; ) &quot;02-88-88-FF-FE-88-88-88&quot; iex&gt; eui64_encode ( &quot;0288.8888.8888&quot; ) &quot;00-88-88-FF-FE-88-88-88&quot; iex&gt; eui64_encode ( { 0x00 , 0x88 , 0x88 , 0x88 , 0x88 , 0x88 } ) { 0x02 , 0x88 , 0x88 , 0xFF , 0xFE , 0x88 , 0x88 , 0x88 } # modified EUI-64 from an existing EUI-64, simply flip the 7th bit iex&gt; eui64_encode ( &quot;01:23:45:67:89:AB:CD:EF&quot; ) &quot;03-23-45-67-89-AB-CD-EF&quot;","ref":"Pfx.html#eui64_encode/1","title":"Pfx.eui64_encode/1","type":"function"},{"doc":"Returns a list of {number, width} -fields for given pfx . If bit_size(pfx.bits) is not a multiple of width , the last {number, width} -tuple, will have a smaller width. Examples iex&gt; fields ( &quot;10.11.12.13&quot; , 8 ) [ { 10 , 8 } , { 11 , 8 } , { 12 , 8 } , { 13 , 8 } ] iex&gt; fields ( { 10 , 11 , 12 , 13 } , 8 ) [ { 10 , 8 } , { 11 , 8 } , { 12 , 8 } , { 13 , 8 } ] iex&gt; fields ( { { 10 , 11 , 12 , 0 } , 24 } , 8 ) [ { 10 , 8 } , { 11 , 8 } , { 12 , 8 } ] iex&gt; fields ( % Pfx { bits : &lt;&lt; 10 , 11 , 12 , 13 &gt;&gt; , maxlen : 32 } , 8 ) [ { 10 , 8 } , { 11 , 8 } , { 12 , 8 } , { 13 , 8 } ] # pfx.bits is not a multiple of 8, hence the {0, 1} at the end iex&gt; fields ( &quot;10.11.12.0/25&quot; , 8 ) [ { 10 , 8 } , { 11 , 8 } , { 12 , 8 } , { 0 , 1 } ] iex&gt; new ( &lt;&lt; 0xacdc :: 16 &gt;&gt; , 128 ) |&gt; fields ( 4 ) [ { 10 , 4 } , { 12 , 4 } , { 13 , 4 } , { 12 , 4 } ] # only 1 field with less bits than given width of 64 iex&gt; new ( &lt;&lt; 255 , 255 &gt;&gt; , 32 ) |&gt; fields ( 64 ) [ { 65535 , 16 } ]","ref":"Pfx.html#fields/2","title":"Pfx.fields/2","type":"function"},{"doc":"Returns the first full length prefix from the set represented by pfx . Examples iex&gt; first ( &quot;10.10.10.1/24&quot; ) &quot;10.10.10.0&quot; iex&gt; first ( &quot;acdc:1976::/32&quot; ) &quot;acdc:1976:0:0:0:0:0:0&quot; # a full address is its own this-network iex&gt; first ( { 10 , 10 , 10 , 1 } ) { 10 , 10 , 10 , 1 } iex&gt; first ( { { 10 , 10 , 10 , 1 } , 24 } ) { { 10 , 10 , 10 , 0 } , 32 } iex&gt; first ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } iex&gt; first ( % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1976 :: 16 &gt;&gt; , maxlen : 128 } ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1976 :: 16 , 0 :: 96 &gt;&gt; , maxlen : 128 }","ref":"Pfx.html#first/1","title":"Pfx.first/1","type":"function"},{"doc":"Flips a single bit at position in given pfx . A negative position is relative to the end of the pfx.bits bitstring. It is an error to point to a bit outside the range of available bits. Examples iex&gt; flip ( &quot;255.255.254.0&quot; , 23 ) &quot;255.255.255.0&quot; iex&gt; flip ( &quot;255.255.255.0&quot; , - 9 ) &quot;255.255.254.0&quot; # flip the 7th bit iex&gt; flip ( &quot;0088.8888.8888&quot; , 6 ) &quot;02-88-88-88-88-88&quot; iex&gt; flip ( { 1 , 2 , 3 , 0 } , 24 ) { 1 , 2 , 3 , 128 } # flip last bit iex&gt; flip ( { { 1 , 2 , 3 , 128 } , 25 } , 24 ) { { 1 , 2 , 3 , 0 } , 25 } iex&gt; flip ( { { 1 , 2 , 3 , 128 } , 25 } , - 1 ) { { 1 , 2 , 3 , 0 } , 25 } iex&gt; flip ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 1 :: 1 &gt;&gt; , maxlen : 32 } , - 1 ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 0 :: 1 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#flip/2","title":"Pfx.flip/2","type":"function"},{"doc":"Formats pfx as a string, using several options: :width , field width (default 8) :base , howto turn a field into a string (default 10, use 16 for hex numbers) :unit , how many fields go into 1 section (default 1) :ssep , howto join the sections together (default &quot;.&quot;) :lsep , howto join a mask if required (default &quot;/&quot;) :mask , whether to add a mask (default false) :reverse , whether to reverse fields before grouping/joining (default false) :padding , whether to pad out the pfx.bits (default true) The defaults are geared towards IPv4 prefixes, but the options should be able to accomodate other domains as well. Notes: the prefix.bits -length is omitted if equal to the prefix.bits -size domain specific submodules probably implement their own formatter. Examples iex&gt; format ( % Pfx { bits : &lt;&lt; 10 , 11 , 12 &gt;&gt; , maxlen : 32 } ) &quot;10.11.12.0/24&quot; iex&gt; format ( { { 10 , 11 , 12 , 0 } , 24 } ) &quot;10.11.12.0/24&quot; iex&gt; format ( { 10 , 11 , 12 , 0 } ) &quot;10.11.12.0&quot; # non-sensical, but there you go iex&gt; format ( &quot;10.11.12.0/24&quot; ) &quot;10.11.12.0/24&quot; # bitstring, note that mask is applied when new creates the `pfx` iex&gt; format ( &quot;1.2.3.4/24&quot; , width : 1 , base : 2 , unit : 8 , mask : false ) &quot;00000001.00000010.00000011.00000000&quot; # mask not appended as its redundant for a full-sized prefix iex&gt; format ( % Pfx { bits : &lt;&lt; 10 , 11 , 12 , 13 &gt;&gt; , maxlen : 32 } ) &quot;10.11.12.13&quot; iex&gt; pfx = new ( &lt;&lt; 0xacdc :: 16 , 0x1976 :: 16 &gt;&gt; , 128 ) iex&gt; format ( pfx , width : 16 , base : 16 , ssep : &quot;:&quot; ) &quot;acdc:1976:0:0:0:0:0:0/32&quot; # # similar, but grouping 4 fields, each 4 bits wide, into a single section # iex&gt; format ( pfx , width : 4 , base : 16 , unit : 4 , ssep : &quot;:&quot; ) &quot;acdc:1976:0000:0000:0000:0000:0000:0000/32&quot; # # this time, omit the acutal pfx length # iex&gt; format ( pfx , width : 16 , base : 16 , ssep : &quot;:&quot; , mask : false ) &quot;acdc:1976:0:0:0:0:0:0&quot; # # ptr for IPv6 using the nibble format: # - dot-separated reversal of all hex digits in the expanded address # iex&gt; pfx ...&gt; |&gt; format ( width : 4 , base : 16 , mask : false , reverse : true ) ...&gt; |&gt; String . downcase ( ) ...&gt; |&gt; ( fn x -&gt; &quot; \#{ x } .ip6.arpa.&quot; end ) . ( ) &quot;0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.6.7.9.1.c.d.c.a.ip6.arpa.&quot; # turn off padding to get reverse zone dns ptr record iex&gt; new ( &lt;&lt; 10 , 11 , 12 &gt;&gt; , 32 ) ...&gt; |&gt; format ( padding : false , reverse : true , mask : false ) ...&gt; |&gt; ( &amp; &quot; \#{ &amp;1 } .in-addr.arpa.&quot; ) . ( ) &quot;12.11.10.in-addr.arpa.&quot;","ref":"Pfx.html#format/2","title":"Pfx.format/2","type":"function"},{"doc":"Creates a Pfx struct for given hexadecimal string . This always returns a Pfx.t/0 struct. A list of punctuation characters can be supplied as a second argument and defaults to [?:, ?-, ?.] . Note that '/' should not be used as that separates the mask from the rest of the binary. Punctuation characters are simply ignored and no positional checks are performed. Contrary to Pfx.from_mac/1 , this function turns a random hexadecimal into a prefix. Do not use this to create a prefix out of an IPv6 address. Examples iex&gt; from_hex ( &quot;1-2:3.4:a-bcdef&quot; ) % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0xAB , 0xCD , 0xEF &gt;&gt; , maxlen : 40 } iex&gt; from_hex ( &quot;1-2:3.4:a-bcdef/16&quot; ) % Pfx { bits : &lt;&lt; 0x12 , 0x34 &gt;&gt; , maxlen : 40 } iex&gt; from_hex ( &quot;1|2|3|A|B|C|DEF&quot; , [ ?| ] ) % Pfx { bits : &lt;&lt; 0x12 , 0x3A , 0xBC , 0xDE , 0xF :: 4 &gt;&gt; , maxlen : 36 } iex&gt; from_hex ( &quot;ABC&quot; ) % Pfx { bits : &lt;&lt; 0xAB , 0xC :: 4 &gt;&gt; , maxlen : 12 } # not for IPv6 addresses .. iex&gt; from_hex ( &quot;2001::1&quot; ) % Pfx { bits : &lt;&lt; 0x20 , 0x01 , 0x1 :: 4 &gt;&gt; , maxlen : 20 }","ref":"Pfx.html#from_hex/2","title":"Pfx.from_hex/2","type":"function"},{"doc":"Creates a Pfx struct from a EUI48/64 strings or tuples. Parsing strings is somewhat relaxed since punctuation characters are interchangeable as long as their positions are correct. Note that new/1 tries to parse binaries as IP prefixes first and would turn an EUI-64 using &quot;:&quot; for punctuation into an IPv6 address. Similarly, a 8-element tuple is seen as IPv6 address. Hence, if you really need to parse EUI-64 binaries with &quot;:&quot;, or have EUI-48/64 tuples, use this function. from_mac/1 also accepts a Pfx struct, but only if its maxlen is either 48 or 64 . If not, an ArgumentError is raised. Examples iex&gt; from_mac ( &quot;11:22:33:44:55:66&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 &gt;&gt; , maxlen : 48 } iex&gt; from_mac ( &quot;11-22-33-44-55-66&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 &gt;&gt; , maxlen : 48 } iex&gt; from_mac ( &quot;1122.3344.5566&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 &gt;&gt; , maxlen : 48 } iex&gt; from_mac ( { 0xaa , 0xbb , 0xcc , 0xdd , 0xee , 0xff } ) % Pfx { bits : &lt;&lt; 0xaa , 0xbb , 0xcc , 0xdd , 0xee , 0xff &gt;&gt; , maxlen : 48 } # keep the OUI iex&gt; from_mac ( { { 0xaa , 0xbb , 0xcc , 0xdd , 0xee , 0xff } , 24 } ) % Pfx { bits : &lt;&lt; 0xaa , 0xbb , 0xcc &gt;&gt; , maxlen : 48 } iex&gt; from_mac ( &quot;11:22:33:44:55:66/24&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 &gt;&gt; , maxlen : 48 } # a EUI-64 iex&gt; from_mac ( &quot;11-22-33-44-55-66-77-88&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; from_mac ( &quot;11:22:33:44:55:66:77:88&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; from_mac ( &quot;11:22:33:44:55:66:77:88/24&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 &gt;&gt; , maxlen : 64 } iex&gt; from_mac ( { 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 } ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; from_mac ( { { 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 } , 24 } ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 &gt;&gt; , maxlen : 64 } # note: from_mac reads nibbles so each address element must be 2 nibbles (!) iex&gt; from_mac ( &quot;01:02:03:04:05:06:07:08&quot; ) % Pfx { bits : &lt;&lt; 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 &gt;&gt; , maxlen : 64 } # mix and match # &quot;:&quot; and &quot;-&quot; are interchangeable iex&gt; from_mac ( &quot;11:22-33:44-55:66&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 &gt;&gt; , maxlen : 48 }","ref":"Pfx.html#from_mac/1","title":"Pfx.from_mac/1","type":"function"},{"doc":"Returns the nth full length prefix in given pfx . Note that offset nth wraps around. See Pfx.member/2 . Examples iex&gt; host ( &quot;10.10.10.0/24&quot; , 128 ) &quot;10.10.10.128&quot; iex&gt; host ( { { 10 , 10 , 10 , 0 } , 24 } , 128 ) { { 10 , 10 , 10 , 128 } , 32 } iex&gt; host ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 128 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 128 &gt;&gt; , maxlen : 32 } # wraps around iex&gt; host ( &quot;10.10.10.0/24&quot; , 256 ) &quot;10.10.10.0&quot;","ref":"Pfx.html#host/2","title":"Pfx.host/2","type":"function"},{"doc":"Returns a list of address prefixes for given pfx . Examples iex&gt; hosts ( &quot;10.10.10.0/30&quot; ) [ &quot;10.10.10.0&quot; , &quot;10.10.10.1&quot; , &quot;10.10.10.2&quot; , &quot;10.10.10.3&quot; ] iex&gt; hosts ( { { 10 , 10 , 10 , 0 } , 30 } ) [ { { 10 , 10 , 10 , 0 } , 32 } , { { 10 , 10 , 10 , 1 } , 32 } , { { 10 , 10 , 10 , 2 } , 32 } , { { 10 , 10 , 10 , 3 } , 32 } ] iex&gt; hosts ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 :: 6 &gt;&gt; , maxlen : 32 } ) [ % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 3 &gt;&gt; , maxlen : 32 } ]","ref":"Pfx.html#hosts/1","title":"Pfx.hosts/1","type":"function"},{"doc":"Inserts bits into pfx -s bitstring, starting at position . The resulting bitstring is silently clipped to the pfx.maxlen . Valid bit positions are bits_size(pfx.bits) .. min(pfx.maxlen-1, bit_size(pfx.bits)) . A position of 0 will prepend the bits , while a position of bit_size(pfx.bits) will append the bits as long as the prefix is not a full length prefix already. A negative position is taken relative to the end. Note that this cannot be used for appending bits, since -1 refers to the last actual bit and there is no such thing as -0 .. Examples # prepend bits iex&gt; insert ( &quot;0.0.0.0/0&quot; , &lt;&lt; 255 &gt;&gt; , 0 ) &quot;255.0.0.0/8&quot; # append bits iex&gt; insert ( &quot;255.255.0.0/16&quot; , &lt;&lt; 255 &gt;&gt; , 16 ) &quot;255.255.255.0/24&quot; # cannot append to a full prefix, positions go from 0..31 iex&gt; insert ( &quot;1.2.3.4&quot; , &lt;&lt; 255 &gt;&gt; , 32 ) ** (ArgumentError) invalid bit position: 32 # but inserting inside the bitstring, is ok iex&gt; insert ( &quot;1.2.3.4&quot; , &lt;&lt; 255 &gt;&gt; , 16 ) &quot;1.2.255.3&quot; # turn EUI48 into a modified EUI64 iex&gt; new ( &quot;0088.8888.8888&quot; ) ...&gt; |&gt; new ( 64 ) ...&gt; |&gt; flip ( 6 ) ...&gt; |&gt; insert ( &lt;&lt; 0xFF , 0xFE &gt;&gt; , 24 ) % Pfx { bits : &lt;&lt; 0x02 , 0x88 , 0x88 , 0xFF , 0xFE , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 64 } # sliently clips to pfx&#39;s maxlen iex&gt; insert ( &quot;1.2.3.0/24&quot; , &lt;&lt; 255 , 255 , 255 , 255 &gt;&gt; , 0 ) &quot;255.255.255.255&quot; iex&gt; insert ( &quot;1.2.3.0/24&quot; , &lt;&lt; 255 , 255 , 255 , 255 &gt;&gt; , 24 ) &quot;1.2.3.255&quot;","ref":"Pfx.html#insert/3","title":"Pfx.insert/3","type":"function"},{"doc":"Returns the inverted mask for given pfx . The result is always a full length prefix. Examples iex&gt; inv_mask ( &quot;10.10.10.0/25&quot; ) &quot;0.0.0.127&quot; iex&gt; inv_mask ( { 10 , 10 , 10 , 0 } ) { 0 , 0 , 0 , 0 } iex&gt; inv_mask ( { { 10 , 10 , 10 , 0 } , 25 } ) { { 0 , 0 , 0 , 127 } , 32 } iex&gt; inv_mask ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 :: 1 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 0 , 0 , 0 , 127 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#inv_mask/1","title":"Pfx.inv_mask/1","type":"function"},{"doc":"Guard that ensures both prefixes are valid and comparable (same maxlen).","ref":"Pfx.html#is_comparable/2","title":"Pfx.is_comparable/2","type":"macro"},{"doc":"Guard that ensures a given pfx is actually valid. it is a Pfx.t/0 struct, pfx.maxlen is a non_neg_integer/0 , bit_size(pfx.bits) &lt;= pfx.maxlen","ref":"Pfx.html#is_pfx/1","title":"Pfx.is_pfx/1","type":"macro"},{"doc":"Keeps count msb bits of given pfx . If count exceeds the actual number of bits in pfx.bits , it keeps all bits. Examples iex&gt; keep ( &quot;2001:0db8:85a3:0000:0000:8a2e:0370:7334&quot; , 64 ) &quot;2001:db8:85a3:0:0:0:0:0/64&quot; iex&gt; keep ( &quot;1.2.3.0/31&quot; , 30 ) &quot;1.2.3.0/30&quot; iex&gt; keep ( &quot;1.2.3.2/31&quot; , 30 ) &quot;1.2.3.0/30&quot; iex&gt; keep ( &quot;1.2.3.128/25&quot; , 24 ) &quot;1.2.3.0/24&quot; iex&gt; keep ( &quot;1.2.3.0/24&quot; , 512 ) &quot;1.2.3.0/24&quot; iex&gt; keep ( { 1 , 2 , 3 , 4 } , 24 ) { 1 , 2 , 3 , 0 } iex&gt; keep ( { { 1 , 2 , 3 , 4 } , 32 } , 16 ) { { 1 , 2 , 0 , 0 } , 16 } iex&gt; keep ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } , 16 ) % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#keep/2","title":"Pfx.keep/2","type":"function"},{"doc":"Returns the last full length prefix from the set represented by pfx . Examples iex&gt; last ( &quot;10.10.0.0/16&quot; ) &quot;10.10.255.255&quot; # a full address is its own last address iex&gt; last ( { 10 , 10 , 10 , 1 } ) { 10 , 10 , 10 , 1 } iex&gt; last ( { { 10 , 10 , 10 , 1 } , 30 } ) { { 10 , 10 , 10 , 3 } , 32 } iex&gt; last ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 255 &gt;&gt; , maxlen : 32 } iex&gt; last ( % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1976 :: 16 &gt;&gt; , maxlen : 128 } ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1976 :: 16 , - 1 :: 96 &gt;&gt; , maxlen : 128 } iex&gt; last ( &quot;acdc:1976::/112&quot; ) &quot;acdc:1976:0:0:0:0:0:ffff&quot;","ref":"Pfx.html#last/1","title":"Pfx.last/1","type":"function"},{"doc":"Returns a map with link-local address components for given pfx . Returns nil if pfx is not link-local as per rfc3927 or rfc4291 Examples iex&gt; x = link_local ( &quot;169.254.128.233&quot; ) iex&gt; x %{ digits : { 169 , 254 , 128 , 233 } , prefix : &quot;169.254.0.0/16&quot; , ifaceID : 33001 , address : &quot;169.254.128.233&quot; } # iex&gt; host ( x . prefix , x . ifaceID ) &quot;169.254.128.233&quot; iex&gt; y = link_local ( &quot;fe80::acdc:1976&quot; ) iex&gt; y %{ preamble : 1018 , prefix : &quot;fe80:0:0:0:0:0:0:0/64&quot; , ifaceID : 2900105590 , address : &quot;fe80::acdc:1976&quot; } # iex&gt; host ( y . prefix , y . ifaceID ) &quot;fe80:0:0:0:0:0:acdc:1976&quot;","ref":"Pfx.html#link_local/1","title":"Pfx.link_local/1","type":"function"},{"doc":"Returns true if pfx is a link-local prefix, false otherwise Link local prefixes include: 0.0.0.0/8 , rfc1122 , 'this-network' 255.255.255.255/32 , rfc1f22 , limited broadcast 169.254.0.0/16 , rfc3927 , link-local (see examples) fe80::/64 , rfc4291 , link-local Examples # first 256 addresses are reserved iex&gt; link_local? ( &quot;169.254.0.0&quot; ) false # last 256 addresses are reserved iex&gt; link_local? ( &quot;169.254.255.0&quot; ) false # rest is considered link local iex&gt; link_local? ( &quot;169.254.1.0&quot; ) true iex&gt; link_local? ( &quot;169.254.254.255&quot; ) true iex&gt; link_local? ( &quot;0.0.0.0&quot; ) true iex&gt; link_local? ( &quot;0.255.255.255&quot; ) true iex&gt; link_local? ( { 0 , 255 , 255 , 255 } ) true iex&gt; link_local? ( &quot;fe80::acdc:1975&quot; ) true iex&gt; link_local? ( &quot;1.1.1.1&quot; ) false # bad prefix iex&gt; link_local? ( &quot;10.10.10.256&quot; ) false","ref":"Pfx.html#link_local?/1","title":"Pfx.link_local?/1","type":"function"},{"doc":"Returns a representation of pfx (a Pfx.t/0 -struct) in the form of original . The exact original is not required, the pfx is transformed by the shape of the original argument: string vs two-element tuple vs tuple. If none of the three shapes match, the pfx is returned unchanged. This is used to allow results to be the same shape as their (first) argument that needed to turn into a Pfx.t/0 for some calculation. Note that when turning a prefix into a address-tuple, an address-tuple comes out which is the first full length prefix in the set represented by pfx . Examples # original is a string iex&gt; marshall ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , &quot;any string really&quot; ) &quot;1.1.1.0/24&quot; # original is any two-element tuple iex&gt; marshall ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , { 0 , 0 } ) { { 1 , 1 , 1 , 0 } , 24 } # original is any other tuple, actually turns prefix into this-network address iex&gt; marshall ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , { } ) { 1 , 1 , 1 , 0 } # original is a Pfx struct iex&gt; marshall ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; &gt;&gt; , maxlen : 0 } ) % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } iex&gt; marshall ( new ( &quot;1.1.1.1&quot; ) , { } ) { 1 , 1 , 1 , 1 }","ref":"Pfx.html#marshall/2","title":"Pfx.marshall/2","type":"function"},{"doc":"Returns the mask for given pfx . The result is always a full length prefix. Examples iex&gt; mask ( &quot;10.10.10.0/25&quot; ) &quot;255.255.255.128&quot; iex&gt; mask ( { 10 , 10 , 10 , 0 } ) { 255 , 255 , 255 , 255 } iex&gt; mask ( { { 10 , 10 , 10 , 0 } , 25 } ) { { 255 , 255 , 255 , 128 } , 32 } iex&gt; mask ( &quot;acdc:1976::/32&quot; ) &quot;ffff:ffff:0:0:0:0:0:0&quot; # some prefix with some other maxlen iex&gt; mask ( % Pfx { bits : &lt;&lt; 10 , 10 , 0 :: 1 &gt;&gt; , maxlen : 20 } ) % Pfx { bits : &lt;&lt; 255 , 255 , 8 :: 4 &gt;&gt; , maxlen : 20 }","ref":"Pfx.html#mask/1","title":"Pfx.mask/1","type":"function"},{"doc":"Applies mask to given prefix . The mask is applied through a bitwise AND after which the result is trimmed to the size of mask . Both prefix and mask do not need to be full length prefixes. Options include: :inv_mask , if true the mask is inverted before applying it (default: false ) :trim , if false the result is not trimmed to the size of mask (default: true ) Note that both prefix and mask must be of the same type (same maxlen). Examples # trims result by default iex&gt; mask ( &quot;1.1.1.1&quot; , &quot;255.255.255.0&quot; ) &quot;1.1.1.0/24&quot; iex&gt; mask ( &quot;1.1.1.1&quot; , &quot;255.255.255.0&quot; ) |&gt; first ( ) &quot;1.1.1.0&quot; # same as above iex&gt; mask ( &quot;1.1.1.1&quot; , &quot;255.255.255.0&quot; , trim : false ) &quot;1.1.1.0&quot; # mirror representation iex&gt; mask ( { { 1 , 1 , 1 , 1 } , 32 } , { 255 , 255 , 255 , 0 } ) { { 1 , 1 , 1 , 0 } , 24 } iex&gt; mask ( &quot;1.1.1.1&quot; , &quot;255.0.0.255&quot; ) &quot;1.0.0.1&quot; # mask need not be full length prefix iex&gt; mask ( &quot;1.1.1.1&quot; , &quot;255.255.0.0/16&quot; ) &quot;1.1.0.0/16&quot; iex&gt; mask ( &quot;1.1.1.1&quot; , &quot;255.255.0.0/16&quot; , trim : false ) &quot;1.1.0.0&quot; # neither does prefix iex&gt; mask ( &quot;1.1.1.0/24&quot; , &quot;255.255.0.0/16&quot; ) &quot;1.1.0.0/16&quot; # no trim, so prefix length stays 24 bits iex&gt; mask ( &quot;1.1.1.0/24&quot; , &quot;255.255.0.0/16&quot; , trim : false ) &quot;1.1.0.0/24&quot; # inverted mask iex&gt; mask ( &quot;10.16.0.0&quot; , &quot;0.3.255.255&quot; , inv_mask : true ) ...&gt; |&gt; ( fn x -&gt; { x , first ( x ) , last ( x ) } end ) . ( ) { &quot;10.16.0.0/14&quot; , &quot;10.16.0.0&quot; , &quot;10.19.255.255&quot; }","ref":"Pfx.html#mask/3","title":"Pfx.mask/3","type":"function"},{"doc":"Returns the nth -member of a given pfx . A prefix represents a range of (possibly longer) prefixes which can be seen as members of the prefix. So a prefix of n -bits long represents: 1 prefix of n -bits long (i.e. itself), 2 prefixes of n+1 -bits long, 4 prefixes of n+2 -bits long .. 2^w prefixes of n+w -bits long where n+w &lt;= pfx.maxlen . Not specifying a width assumes the maximum width available. If a width is specified, the nth -offset is added to the prefix as a number width -bits wide. This wraps around the available address space. Examples iex&gt; member ( &quot;10.10.10.0/24&quot; , 255 ) &quot;10.10.10.255&quot; # wraps around iex&gt; member ( &quot;10.10.10.0/24&quot; , 256 ) &quot;10.10.10.0&quot; iex&gt; member ( { { 10 , 10 , 10 , 0 } , 24 } , 255 ) { { 10 , 10 , 10 , 255 } , 32 } iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 0 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 255 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 255 &gt;&gt; , maxlen : 32 } # wraps around iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 256 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , - 1 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 255 &gt;&gt; , maxlen : 32 } # a full prefix always returns itself iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 0 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 10 &gt;&gt; , maxlen : 32 } iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 3 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 10 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#member/2","title":"Pfx.member/2","type":"function"},{"doc":"Returns the nth member in the set represented by pfx , using width bits. Examples iex&gt; member ( &quot;10.10.10.0/24&quot; , 1 , 2 ) &quot;10.10.10.64/26&quot; iex&gt; member ( &quot;10.10.10.0/24&quot; , 2 , 2 ) &quot;10.10.10.128/26&quot; iex&gt; member ( { { 10 , 10 , 10 , 0 } , 24 } , 2 , 2 ) { { 10 , 10 , 10 , 128 } , 26 } # the first member that is 2 bits longer iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 0 , 2 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 :: 2 &gt;&gt; , maxlen : 32 } # the second member that is 2 bits longer iex&gt; member ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , 1 , 2 ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 1 :: 2 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#member/3","title":"Pfx.member/3","type":"function"},{"doc":"Returns true is prefix pfx1 is a member of prefix pfx2 If either prfx1 or pfx2 is invalid or they are of different types, member? simply returns false. Examples iex&gt; member? ( &quot;10.10.10.10&quot; , &quot;10.0.0.0/8&quot; ) true iex&gt; member? ( { 10 , 10 , 10 , 10 } , &quot;10.0.0.0/8&quot; ) true iex&gt; member? ( { { 10 , 10 , 10 , 10 } , 24 } , &quot;10.0.0.0/8&quot; ) true iex&gt; member? ( { { 11 , 0 , 0 , 0 } , 8 } , { { 10 , 0 , 0 , 0 } , 8 } ) false iex&gt; member? ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 10 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 &gt;&gt; , maxlen : 32 } ) true # bad prefix iex&gt; member? ( &quot;10.10.10.10&quot; , &quot;10.10.10.256/24&quot; ) false","ref":"Pfx.html#member?/2","title":"Pfx.member?/2","type":"function"},{"doc":"Returns true is pfx is a multicast prefix, false otherwise Checks if pfx given is a member of: 224.0.0.0/4 rfc1112 . ff00:/8 rfc4921 Examples iex&gt; multicast? ( &quot;224.0.0.1&quot; ) true iex&gt; multicast? ( &quot;ff02::1&quot; ) true iex&gt; multicast? ( { { 224 , 0 , 0 , 1 } , 32 } ) true iex&gt; multicast? ( { 224 , 0 , 0 , 1 } ) true iex&gt; multicast? ( % Pfx { bits : &lt;&lt; 224 , 0 , 0 , 1 &gt;&gt; , maxlen : 32 } ) true iex&gt; multicast? ( &quot;1.1.1.1&quot; ) false # bad prefix iex&gt; multicast? ( &quot;224.0.0.256&quot; ) false","ref":"Pfx.html#multicast?/1","title":"Pfx.multicast?/1","type":"function"},{"doc":"Returns a map with multicast address components for given pfx . Address components are parsed according to: rfc1112 - Host Extensions for IP Multicasting rfc4291 - IP Version 6 Addressing Architecture Rfc specific fields are put in their own map under the rfc -key. rfc6034 - Unicast-Prefix-Based IPv4 Multicast Addresses rfc3180 - GLOP Addressing in 233/8 rfc3956 - Embedding the Rendezvous Point (RP) Address in an IPv6 Multicast Address rfc3306 - Unicast-Prefix-based IPv6 Multicast Addresses Note that for unicast-prefix-based IPv4 multicast addresses, the unicast prefix is always taken to be 24 bits long. That should still allow for identification of the origin by looking up the assigned unicast address space that includes the /24. Examples iex&gt; multicast_decode ( &quot;234.192.0.2&quot; ) %{ multicast_address : &quot;234.192.0.2&quot; , protocol : :ipv4 , rfc : %{ group_id : 2 , multicast_prefix : &quot;234.0.0.0/8&quot; , rfc : 6034 , unicast_prefix : &quot;192.0.2.0/24&quot; } } iex&gt; Pfx . multicast_decode ( &quot;FF32:0030:3FFE:FFFF:0001::/96&quot; ) %{ flags : { 0 , 0 , 1 , 1 } , multicast_address : &quot;ff32:30:3ffe:ffff:1:0:0:0/96&quot; , multicast_prefix : &quot;ff30:0:0:0:0:0:0:0/12&quot; , protocol : :ipv6 , rfc : %{ group_id : 0 , unicast_prefix : &quot;3ffe:ffff:1:0:0:0:0:0/48&quot; , plen : 48 , reserved : 0 , rfc : 3306 } , scope : 2 }","ref":"Pfx.html#multicast_decode/1","title":"Pfx.multicast_decode/1","type":"function"},{"doc":"Returns true if pfx is matched by the Well-Known Prefixes defined in rfc6053 and rfc8215 , false otherwise. Note that organisation specific prefixes might still be used for nat64. Examples iex&gt; nat64? ( &quot;64:ff9b::10.10.10.10&quot; ) true iex&gt; nat64? ( &quot;64:ff9b:1::10.10.10.10&quot; ) true iex&gt; nat64? ( { { 0x64 , 0xff9b , 0 , 0 , 0 , 0 , 0x1010 , 0x1010 } , 128 } ) true iex&gt; nat64? ( { 0x64 , 0xff9b , 0 , 0 , 0 , 0 , 0x1010 , 0x1010 } ) true iex&gt; nat64? ( % Pfx { bits : &lt;&lt; 0x64 :: 16 , 0xff9b :: 16 , 0 :: 64 , 0x1010 :: 16 , 0x1010 :: 16 &gt;&gt; , maxlen : 128 } ) true # illegal/bad prefix iex&gt; nat64? ( &quot;64:ff9b:1::10.10.10.256&quot; ) false","ref":"Pfx.html#nat64?/1","title":"Pfx.nat64?/1","type":"function"},{"doc":"Returns the embedded IPv4 address of a nat64 pfx The pfx prefix should be a full IPv6 address. The len defaults to 96 , but if specified it should be one of [96, 64, 56, 48, 40, 32]. Examples iex&gt; nat64_decode ( &quot;64:ff9b::10.10.10.10&quot; ) &quot;10.10.10.10&quot; iex&gt; nat64_decode ( &quot;64:ff9b:1:0a0a:000a:0a00::&quot; , 48 ) &quot;10.10.10.10&quot; # from rfc6052, section 2.4 iex&gt; nat64_decode ( &quot;2001:db8:c000:221::&quot; , 32 ) &quot;192.0.2.33&quot; iex&gt; nat64_decode ( &quot;2001:db8:1c0:2:21::&quot; , 40 ) &quot;192.0.2.33&quot; iex&gt; nat64_decode ( &quot;2001:db8:122:c000:2:2100::&quot; , 48 ) &quot;192.0.2.33&quot; iex&gt; nat64_decode ( &quot;2001:db8:122:3c0:0:221::&quot; , 56 ) &quot;192.0.2.33&quot; iex&gt; nat64_decode ( &quot;2001:db8:122:344:c0:2:2100::&quot; , 64 ) &quot;192.0.2.33&quot; iex&gt; nat64_decode ( { 0x2001 , 0xdb8 , 0x122 , 0x344 , 0xC0 , 0x2 , 0x2100 , 0x0 } , 64 ) { 192 , 0 , 2 , 33 } iex&gt; nat64_decode ( { { 0x2001 , 0xdb8 , 0x122 , 0x344 , 0xC0 , 0x2 , 0x2100 , 0x0 } , 128 } , 64 ) { { 192 , 0 , 2 , 33 } , 32 } iex&gt; nat64_decode ( &quot;2001:db8:122:344::192.0.2.33&quot; , 96 ) &quot;192.0.2.33&quot; iex&gt; nat64_decode ( &quot;2001:db8:122:344::192.0.2.33&quot; , 90 ) ** (ArgumentError) nat64 prefix length not in [96, 64, 56, 48, 40, 32], got 90","ref":"Pfx.html#nat64_decode/2","title":"Pfx.nat64_decode/2","type":"function"},{"doc":"Returns an IPv4 embedded IPv6 address for given pfx6 and pfx4 . The length of the pfx6.bits should be one of [96, 64, 56, 48, 40, 32] as defined in rfc6052 . The pfx4 prefix should be a full address. Examples iex&gt; nat64_encode ( &quot;2001:db8:100::/40&quot; , &quot;192.0.2.33&quot; ) &quot;2001:db8:1c0:2:21:0:0:0&quot; iex&gt; nat64_encode ( &quot;2001:db8:122::/48&quot; , &quot;192.0.2.33&quot; ) &quot;2001:db8:122:c000:2:2100:0:0&quot; iex&gt; nat64_encode ( &quot;2001:db8:122:300::/56&quot; , &quot;192.0.2.33&quot; ) &quot;2001:db8:122:3c0:0:221:0:0&quot; iex&gt; nat64_encode ( &quot;2001:db8:122:344::/64&quot; , &quot;192.0.2.33&quot; ) &quot;2001:db8:122:344:c0:2:2100:0&quot; iex&gt; nat64_encode ( &quot;2001:db8:122:344::/96&quot; , &quot;192.0.2.33&quot; ) &quot;2001:db8:122:344:0:0:c000:221&quot; iex&gt; nat64_encode ( { { 0x2001 , 0xdb8 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 } , &quot;192.0.2.33&quot; ) { { 0x2001 , 0xdb8 , 0xc000 , 0x221 , 0 , 0 , 0 , 0 } , 128 } iex&gt; nat64_encode ( % Pfx { bits : &lt;&lt; 0x2001 :: 16 , 0xdb8 :: 16 &gt;&gt; , maxlen : 128 } , &quot;192.0.2.33&quot; ) % Pfx { bits : &lt;&lt; 0x2001 :: 16 , 0xdb8 :: 16 , 0xc000 :: 16 , 0x221 :: 16 , 0 :: 64 &gt;&gt; , maxlen : 128 } iex&gt; nat64_encode ( &quot;2001:db8::/32&quot; , &quot;192.0.2.33&quot; ) &quot;2001:db8:c000:221:0:0:0:0&quot;","ref":"Pfx.html#nat64_encode/2","title":"Pfx.nat64_encode/2","type":"function"},{"doc":"Returns the neighboring prefix for pfx , such that both can be combined in a supernet. Examples iex&gt; neighbor ( &quot;1.1.1.128/25&quot; ) &quot;1.1.1.0/25&quot; iex&gt; neighbor ( &quot;1.1.1.0/25&quot; ) &quot;1.1.1.128/25&quot; iex&gt; neighbor ( { 1 , 1 , 1 , 1 } ) { 1 , 1 , 1 , 0 } iex&gt; neighbor ( { { 1 , 1 , 1 , 128 } , 25 } ) { { 1 , 1 , 1 , 0 } , 25 } iex&gt; neighbor ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 1 , 1 , 1 , 0 :: 1 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#neighbor/1","title":"Pfx.neighbor/1","type":"function"},{"doc":"Returns the this-network prefix (full address) for given pfx . Yields the same result as Pfx.first/1 , included for nostalgia. Examples iex&gt; network ( &quot;10.10.10.1/24&quot; ) &quot;10.10.10.0&quot; iex&gt; network ( &quot;acdc:1976::/32&quot; ) &quot;acdc:1976:0:0:0:0:0:0&quot; # a full address is its own this-network iex&gt; network ( { 10 , 10 , 10 , 1 } ) { 10 , 10 , 10 , 1 } iex&gt; network ( { { 10 , 10 , 10 , 1 } , 24 } ) { { 10 , 10 , 10 , 0 } , 32 } iex&gt; network ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } iex&gt; network ( % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1976 :: 16 &gt;&gt; , maxlen : 128 } ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1976 :: 16 , 0 :: 96 &gt;&gt; , maxlen : 128 }","ref":"Pfx.html#network/1","title":"Pfx.network/1","type":"function"},{"doc":"Creates a new prefix from address tuples/binaries or raises ArgumentError . Use: a binary in CIDR -notation, a binary in EUI-48 or EUI-64 format (EUI-64 must be using hyphens !) an { ip_address/0 , length }-tuple to truncate the bits to length . an ipv4 or ipv6 ip_address/0 tuple directly for a full address, or a Pfx.t/0 struct to create a new PFx struct. To avoid a possible ArgumentError , use Pfx.parse/1 or Pfx.parse/2 instead. To avoid applying the mask, use Pfx.address/1 then apply Pfx.new/1 if needed. Binaries are processed by :inet.parse_address/1 , so be aware of IPv4 shorthand notations that may yield surprising results, since digits are taken to be: d1.d2.d3.d4 -&gt; d1.d2.d3.d4 (full address) d1.d2.d3 -&gt; d1.d2.0.d3 d1.d2 -&gt; d1.0.0.d2 d1 -&gt; 0.0.0.d1 If :inet.parse_address/1 fails to create an IPv4 or IPv6 address, an attempt is made to parse the binary as an EUI-48 or EUI-64 MAC address. Parsing EUI's is somewhat relaxed, punctuation chars &quot;-&quot;, &quot;:&quot;, &quot;.&quot; are interchangeable, but their positions should be correct. Note that EUI-64's that use &quot;:&quot;-punctuation are indistinguishable from IPv6, e.g. &quot;11:22:33:44:55:66:77:88&quot;. Use from_mac/1 when in doubt about punctuations used while parsing MAC addresses. Examples # from CIDR strings iex&gt; new ( &quot;10.10.0.0&quot; ) % Pfx { bits : &lt;&lt; 10 , 10 , 0 , 0 &gt;&gt; , maxlen : 32 } iex&gt; new ( &quot;10.10.10.10/16&quot; ) % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } # ipv6 string iex&gt; new ( &quot;acdc:1976::/32&quot; ) % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1976 :: 16 &gt;&gt; , maxlen : 128 } # from an {address-tuple, length} iex&gt; new ( { { 0xacdc , 0x1976 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 } ) % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1976 :: 16 &gt;&gt; , maxlen : 128 } iex&gt; new ( { { 10 , 10 , 0 , 0 } , 16 } ) % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } # from an address-tuple iex&gt; new ( { 10 , 10 , 0 , 0 } ) % Pfx { bits : &lt;&lt; 10 , 10 , 0 , 0 &gt;&gt; , maxlen : 32 } # from a struct iex&gt; new ( % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } # 10.10/16 is interpreted as 10.0.0.10/16 (!) iex&gt; new ( &quot;10.10/16&quot; ) % Pfx { bits : &lt;&lt; 10 , 0 &gt;&gt; , maxlen : 32 } # some EUI-48&#39;s iex&gt; new ( &quot;aa:bb:cc:dd:ee:ff&quot; ) % Pfx { bits : &lt;&lt; 0xaa , 0xbb , 0xcc , 0xdd , 0xee , 0xff &gt;&gt; , maxlen : 48 } iex&gt; new ( &quot;aa-bb-cc-dd-ee-ff&quot; ) % Pfx { bits : &lt;&lt; 0xaa , 0xbb , 0xcc , 0xdd , 0xee , 0xff &gt;&gt; , maxlen : 48 } iex&gt; new ( &quot;aabb.ccdd.eeff&quot; ) % Pfx { bits : &lt;&lt; 0xaa , 0xbb , 0xcc , 0xdd , 0xee , 0xff &gt;&gt; , maxlen : 48 } # keep only OUI iex&gt; new ( &quot;aa-bb-cc-dd-ee-ff/24&quot; ) % Pfx { bits : &lt;&lt; 0xaa , 0xbb , 0xcc &gt;&gt; , maxlen : 48 } # some EUI-64&#39;s iex&gt; new ( &quot;11-22-33-44-55-66-77-88&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; new ( &quot;1122.3344.5566.7788&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } # but note the maxlen here ... iex&gt; new ( &quot;11:22:33:44:55:66:77:88&quot; ) % Pfx { bits : &lt;&lt; 0x11 :: 16 , 0x22 :: 16 , 0x33 :: 16 , 0x44 :: 16 , 0x55 :: 16 , 0x66 :: 16 , 0x77 :: 16 , 0x88 :: 16 &gt;&gt; , maxlen : 128 } iex&gt; try do ...&gt; new ( &quot;1.1.1.256&quot; ) ...&gt; rescue ...&gt; x -&gt; Exception . message ( x ) ...&gt; end &quot;expected a ipv4/ipv6 CIDR or EUI-48/64 string, got \\&quot; 1.1.1.256 \\&quot; &quot;","ref":"Pfx.html#new/1","title":"Pfx.new/1","type":"function"},{"doc":"Creates a new Pfx.t/0 -prefix. Create a new prefix from: from a bitstring and a maximum length, truncating the bits as needed, from a Pfx.t/0 prefix and a new maxlen, again truncating as needed, Examples iex&gt; new ( &lt;&lt; 10 , 10 &gt;&gt; , 32 ) % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } iex&gt; new ( &lt;&lt; 10 , 10 &gt;&gt; , 8 ) % Pfx { bits : &lt;&lt; 10 &gt;&gt; , maxlen : 8 } # changing &#39;maxlen&#39; usually changes the prefix&#39; meaning iex&gt; new ( % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 32 } , 128 ) % Pfx { bits : &lt;&lt; 10 , 10 &gt;&gt; , maxlen : 128 }","ref":"Pfx.html#new/2","title":"Pfx.new/2","type":"function"},{"doc":"Pads the bits in pfx on the left to its full length using 0 -bits. Examples iex&gt; padl ( &quot;1.2.0.0/16&quot; ) &quot;0.0.1.2&quot; iex&gt; padl ( { { 1 , 2 , 0 , 0 } , 16 } ) { { 0 , 0 , 1 , 2 } , 32 } iex&gt; padl ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 0 , 0 , 1 , 2 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#padl/1","title":"Pfx.padl/1","type":"function"},{"doc":"Left pad the pfx.bits to its full length using either 0 or 1 -bits. Examples iex&gt; padl ( &quot;1.2.0.0/16&quot; , 1 ) &quot;255.255.1.2&quot; iex&gt; padl ( { { 1 , 2 , 0 , 0 } , 16 } , 1 ) { { 255 , 255 , 1 , 2 } , 32 } iex&gt; padl ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } , 1 ) % Pfx { bits : &lt;&lt; 255 , 255 , 1 , 2 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#padl/2","title":"Pfx.padl/2","type":"function"},{"doc":"Pads the bits in pfx on the Left with n bits of either 0 or 1 's. Examples iex&gt; padl ( &quot;255.255.0.0/16&quot; , 0 , 16 ) &quot;0.0.255.255&quot; iex&gt; padl ( &quot;255.255.0.0/16&quot; , 1 , 16 ) &quot;255.255.255.255&quot; iex&gt; padl ( { { 255 , 255 , 0 , 0 } , 16 } , 0 , 16 ) { { 0 , 0 , 255 , 255 } , 32 } iex&gt; padl ( % Pfx { bits : &lt;&lt; 255 , 255 &gt;&gt; , maxlen : 32 } , 0 , 16 ) % Pfx { bits : &lt;&lt; 0 , 0 , 255 , 255 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#padl/3","title":"Pfx.padl/3","type":"function"},{"doc":"Pads the bits in pfx on the right to its full length using 0 -bits. The result is always a full prefix with maxlen bits. Examples # already a full address iex&gt; padr ( &quot;1.2.3.4&quot; ) &quot;1.2.3.4&quot; # mask applied first, then padded with zero&#39;s iex&gt; padr ( &quot;1.2.3.4/16&quot; ) &quot;1.2.0.0&quot; # mask applied first, than padded with zero&#39;s iex&gt; padr ( { { 1 , 2 , 0 , 0 } , 16 } ) { { 1 , 2 , 0 , 0 } , 32 } iex&gt; padr ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } ) % Pfx { bits : &lt;&lt; 1 , 2 , 0 , 0 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#padr/1","title":"Pfx.padr/1","type":"function"},{"doc":"Pads the bits in pfx on the right to its full length using either 0 or 1 -bits. Examples iex&gt; padr ( &quot;1.2.0.0/16&quot; , 1 ) &quot;1.2.255.255&quot; iex&gt; padr ( { { 1 , 2 , 0 , 0 } , 16 } , 1 ) { { 1 , 2 , 255 , 255 } , 32 } # nothing to padr, already a full prefix iex&gt; padr ( &quot;1.2.0.0&quot; , 1 ) &quot;1.2.0.0&quot; iex&gt; padr ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 32 } , 1 ) % Pfx { bits : &lt;&lt; 1 , 2 , 255 , 255 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#padr/2","title":"Pfx.padr/2","type":"function"},{"doc":"Pads the bits in pfx on the right with n bits of either 0 or 1 's. The result is clipped at maxlen bits without warning. Examples # expand a /16 to a /24 iex&gt; padr ( &quot;255.255.0.0/16&quot; , 0 , 8 ) &quot;255.255.0.0/24&quot; iex&gt; padr ( &quot;255.255.0.0/16&quot; , 1 , 8 ) &quot;255.255.255.0/24&quot; iex&gt; padr ( { { 255 , 255 , 0 , 0 } , 16 } , 1 , 8 ) { { 255 , 255 , 255 , 0 } , 24 } # results are clipped to maxlen iex&gt; padr ( &quot;1.2.0.0/16&quot; , 1 , 512 ) &quot;1.2.255.255&quot; iex&gt; padr ( % Pfx { bits : &lt;&lt; 255 , 255 &gt;&gt; , maxlen : 32 } , 0 , 8 ) % Pfx { bits : &lt;&lt; 255 , 255 , 0 &gt;&gt; , maxlen : 32 } iex&gt; padr ( % Pfx { bits : &lt;&lt; 255 , 255 &gt;&gt; , maxlen : 32 } , 1 , 8 ) % Pfx { bits : &lt;&lt; 255 , 255 , 255 &gt;&gt; , maxlen : 32 }","ref":"Pfx.html#padr/3","title":"Pfx.padr/3","type":"function"},{"doc":"Parses a prefix/0 and returns {:ok, Pfx.t} or {:error, :einvalid} Examples iex&gt; parse ( &quot;1.2.3.4/24&quot; ) { :ok , % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } } iex&gt; parse ( { { 1 , 2 , 3 , 4 } , 24 } ) { :ok , % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } } iex&gt; parse ( { 1 , 2 , 3 , 4 } ) { :ok , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } } iex&gt; parse ( % Pfx { bits : &lt;&lt; 4 , 3 , 2 , 1 &gt;&gt; , maxlen : 32 } ) { :ok , % Pfx { bits : &lt;&lt; 4 , 3 , 2 , 1 &gt;&gt; , maxlen : 32 } } iex&gt; parse ( &quot;1.2.3.4/33&quot; ) { :error , :einvalid } iex&gt; parse ( &quot;acdc:1976::/32&quot; ) { :ok , % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1976 :: 16 &gt;&gt; , maxlen : 128 } } iex&gt; parse ( &quot;DEAD:BEER::/32&quot; ) { :error , :einvalid }","ref":"Pfx.html#parse/1","title":"Pfx.parse/1","type":"function"},{"doc":"Parses a prefix/0 and returns {:ok, Pfx.t} or given default on error. Same as Pfx.parse/1 , but returns given default on error. Examples iex&gt; parse ( &quot;0.0.0.0/32&quot; , :oops ) { :ok , % Pfx { bits : &lt;&lt; 0 , 0 , 0 , 0 &gt;&gt; , maxlen : 32 } } iex&gt; parse ( &quot;0.0.0.0/33&quot; , :oops ) :oops iex&gt; pfx = &quot;0.0.0.256/24&quot; iex&gt; parse ( pfx , { :error , pfx } ) { :error , &quot;0.0.0.256/24&quot; } iex&gt; parse ( &quot;11:22:33:44:55:GG&quot; , { :error , :bad_eui48 } ) { :error , :bad_eui48 }","ref":"Pfx.html#parse/2","title":"Pfx.parse/2","type":"function"},{"doc":"Partitions pfx into a list of new prefixes, each bitlen long. Note that bitlen must be in the range of bit_size(pfx.bits)..pfx.maxlen-1 . Examples # break out the /26&#39;s in a /24 iex&gt; partition ( &quot;10.11.12.0/24&quot; , 26 ) [ &quot;10.11.12.0/26&quot; , &quot;10.11.12.64/26&quot; , &quot;10.11.12.128/26&quot; , &quot;10.11.12.192/26&quot; ] iex&gt; partition ( { { 10 , 11 , 12 , 0 } , 24 } , 26 ) [ { { 10 , 11 , 12 , 0 } , 26 } , { { 10 , 11 , 12 , 64 } , 26 } , { { 10 , 11 , 12 , 128 } , 26 } , { { 10 , 11 , 12 , 192 } , 26 } , ] iex&gt; partition ( % Pfx { bits : &lt;&lt; 10 , 11 , 12 &gt;&gt; , maxlen : 32 } , 26 ) [ % Pfx { bits : &lt;&lt; 10 , 11 , 12 , 0 :: size ( 2 ) &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 11 , 12 , 1 :: size ( 2 ) &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 11 , 12 , 2 :: size ( 2 ) &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 11 , 12 , 3 :: size ( 2 ) &gt;&gt; , maxlen : 32 } ]","ref":"Pfx.html#partition/2","title":"Pfx.partition/2","type":"function"},{"doc":"Returns a list of prefixes that cover the given range of address space. The (inclusive) range can be specified either by: start , stop prefixes, or start , nhosts When using the start , stop -prefixes as a range, both prefixes need to be of the same type, otherwise an argument error is raised. The second form of start , nhosts requires that nhosts does not exceed the addressable capacity of given start prefix's type. If start lies to the right of stop , they are not reversed and the build up of the list of prefixes will wrap around the available address space. If nhosts is negative, start is effectively the last address. Note: any mask information for start or stop prefixes are ignored, if possible. Examples iex&gt; partition_range ( &quot;10.10.10.0&quot; , &quot;10.10.10.130&quot; ) [ &quot;10.10.10.0/25&quot; , &quot;10.10.10.128/31&quot; , &quot;10.10.10.130&quot; ] iex&gt; partition_range ( &quot;10.10.10.0&quot; , 131 ) [ &quot;10.10.10.0/25&quot; , &quot;10.10.10.128/31&quot; , &quot;10.10.10.130&quot; ] iex&gt; partition_range ( &quot;10.10.10.0&quot; , 131 ) ...&gt; |&gt; Enum . map ( &amp; size / 1 ) |&gt; Enum . sum ( ) 131 iex(481)&gt; partition_range ( &quot;acdc::1976&quot; , &quot;acdc::2021&quot; ) [ &quot;acdc:0:0:0:0:0:0:1976/127&quot; , &quot;acdc:0:0:0:0:0:0:1978/125&quot; , &quot;acdc:0:0:0:0:0:0:1980/121&quot; , &quot;acdc:0:0:0:0:0:0:1a00/119&quot; , &quot;acdc:0:0:0:0:0:0:1c00/118&quot; , &quot;acdc:0:0:0:0:0:0:2000/123&quot; , &quot;acdc:0:0:0:0:0:0:2020/127&quot; ] When working with address tuples, the result will be in addres,length-tuples otherwise prefix length information would be lost. # 128 hosts, starting with &quot;10.10.10.128&quot; iex&gt; partition_range ( { 10 , 10 , 10 , 128 } , 128 ) [ { { 10 , 10 , 10 , 128 } , 25 } ] # 1 host, starting with &quot;10.10.10.10&quot; iex&gt; partition_range ( { 10 , 10 , 10 , 10 } , 1 ) [ { { 10 , 10 , 10 , 10 } , 32 } ] # 0 hosts always yield an empty list iex&gt; partition_range ( { 10 , 10 , 10 , 10 } , 0 ) [ ] iex&gt; partition_range ( &quot;10.10.10.10&quot; , 0 ) [ ] # binary format may not show the /len, if it is a full prefix iex&gt; partition_range ( &quot;10.10.10.10&quot; , 1 ) [ &quot;10.10.10.10&quot; ] The range is inclusive, so both start and stop are always included. In fact, start is the first address of the first prefix in the list and stop the last address in the last prefix. iex&gt; partition_range ( &quot;10.10.10.10&quot; , &quot;10.10.10.10&quot; ) [ &quot;10.10.10.10&quot; ] iex&gt; partition_range ( &quot;10.10.10.0&quot; , 512 ) [ &quot;10.10.10.0/23&quot; ] iex&gt; partition_range ( &quot;10.10.10.0&quot; , 131 ) [ &quot;10.10.10.0/25&quot; , &quot;10.10.10.128/31&quot; , &quot;10.10.10.130&quot; ] # negative number means start is actually the last address in the range iex&gt; partition_range ( &quot;10.10.10.130&quot; , - 131 ) [ &quot;10.10.10.0/25&quot; , &quot;10.10.10.128/31&quot; , &quot;10.10.10.130&quot; ] Any mask information, if present, is ignored for both start and stop . Note that when using Pfx.t/0 structs, the mask will already have been applied in which case the address will be the first address in the prefix. iex&gt; partition_range ( &quot;10.10.10.8/24&quot; , 10 ) [ &quot;10.10.10.8/29&quot; , &quot;10.10.10.16/31&quot; ] iex&gt; partition_range ( &quot;10.10.10.10/24&quot; , &quot;10.10.10.17/24&quot; ) [ &quot;10.10.10.10/31&quot; , &quot;10.10.10.12/30&quot; , &quot;10.10.10.16/31&quot; ] # Pfx.new() has already applied the mask iex&gt; start = new ( &quot;10.10.10.10/24&quot; ) iex&gt; stop = new ( &quot;10.10.10.17/24&quot; ) iex&gt; partition_range ( start , stop ) [ % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } ] # so the above is basically the same as: iex&gt; partition_range ( &quot;10.10.10.0&quot; , &quot;10.10.10.0&quot; ) [ &quot;10.10.10.0&quot; ] The list of prefixes is built up starting with start and prefixes are added until it represents the entire range, wrapping the available address space if needed. So if start actually lies to the right of stop , or if nhosts is large enough, wrapping may occur. To avoid wrapping use Pfx.compare/2 to check whether or not to swap start and stop . # happily wraps around address space boundary (so beware) iex&gt; partition_range ( &quot;0.0.0.0&quot; , - 257 ) [ &quot;255.255.255.0/24&quot; , &quot;0.0.0.0&quot; ] iex&gt; partition_range ( &quot;255.255.255.0&quot; , 257 ) [ &quot;255.255.255.0/24&quot; , &quot;0.0.0.0&quot; ] iex&gt; partition_range ( &quot;255.255.255.255&quot; , &quot;0.0.0.255&quot; ) [ &quot;255.255.255.255&quot; , &quot;0.0.0.0/24&quot; ] iex&gt; start = &quot;10.10.255.255&quot; iex&gt; stop = &quot;10.10.0.0&quot; iex&gt; case compare ( start , stop ) do ...&gt; :gt -&gt; partition_range ( stop , start ) ...&gt; _ -&gt; partition_range ( start , stop ) ...&gt; end [ &quot;10.10.0.0/16&quot; ] Finally, the list of prefixes will have varying prefix lengths that ramp up and down. So, when building some sort of access control list, sorting on prefix lengths (ascending, so less specifics come first) may be preferable. iex&gt; partition_range ( &quot;10.10.10.10&quot; , &quot;10.10.10.31&quot; ) [ &quot;10.10.10.10/31&quot; , &quot;10.10.10.12/30&quot; , &quot;10.10.10.16/28&quot; ] iex&gt; partition_range ( &quot;10.10.10.10&quot; , &quot;10.10.10.31&quot; ) ...&gt; |&gt; Enum . sort ( &amp; ( pfxlen ( &amp;1 ) &lt;= pfxlen ( &amp;2 ) ) ) [ &quot;10.10.10.16/28&quot; , &quot;10.10.10.12/30&quot; , &quot;10.10.10.10/31&quot; ] # the above actually converts between string and `t:Pfx.t/0` twice, # to avoid that do something like this: iex&gt; partition_range ( new ( &quot;10.10.10.10&quot; ) , new ( &quot;10.10.10.31&quot; ) ) ...&gt; |&gt; Enum . sort ( &amp; ( pfxlen ( &amp;1 ) &lt;= pfxlen ( &amp;2 ) ) ) ...&gt; |&gt; Enum . map ( &amp; format / 1 ) [ &quot;10.10.10.16/28&quot; , &quot;10.10.10.12/30&quot; , &quot;10.10.10.10/31&quot; ]","ref":"Pfx.html#partition_range/2","title":"Pfx.partition_range/2","type":"function"},{"doc":"Returns the length of the bitstring for given prefix . Examples iex&gt; pfxlen ( &quot;10.10.10.0/24&quot; ) 24 iex&gt; pfxlen ( { { 10 , 10 , 10 , 0 } , 25 } ) 25 iex&gt; pfxlen ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 :: 1 &gt;&gt; , maxlen : 32 } ) 25 iex&gt; pfxlen ( &quot;10.10.10.10&quot; ) 32","ref":"Pfx.html#pfxlen/1","title":"Pfx.pfxlen/1","type":"function"},{"doc":"Removes length bits from pfx -s bitstring, starting at position . A negative position is relative to the end of the pfx.bits -string. Valid range for position is -bit_size(pfx.bits) .. bit_size(pfx.bits)-1 . If length is positive, bits are removed to the right. If it is negative bits are removed going to the left. Notes: length is silently clipped to the maximum number of bits available to remove removing bits from pfx.bits does not change its pfx.maxlen Examples # remove 2nd digit (2) iex&gt; remove ( &quot;1.2.3.4&quot; , 8 , 8 ) &quot;1.3.4.0/24&quot; # remove 25th bit iex&gt; remove ( &quot;1.2.3.128/25&quot; , - 1 , 1 ) &quot;1.2.3.0/24&quot; # remove the FF.FE part iex&gt; remove ( &quot;0288.88FF.FE88.8888&quot; , 24 , 16 ) &quot;02-88-88-88-88-88-00-00/48&quot; # remove 2nd digit (2) iex&gt; remove ( { { 1 , 2 , 3 , 4 } , 32 } , 8 , 8 ) { { 1 , 3 , 4 , 0 } , 24 }","ref":"Pfx.html#remove/3","title":"Pfx.remove/3","type":"function"},{"doc":"Returns another Pfx at distance offset from given pfx . This basically increases or decreases the number represented by the pfx.bits while keeping pfx.maxlen the same. Note that the length of pfx.bits will not change and cycling through all siblings will eventually wrap around. Examples iex&gt; sibling ( &quot;1.2.3.0/24&quot; , - 1 ) &quot;1.2.2.0/24&quot; iex&gt; sibling ( &quot;0.0.0.0&quot; , - 1 ) &quot;255.255.255.255&quot; iex&gt; sibling ( { { 1 , 2 , 3 , 0 } , 24 } , 256 ) { { 1 , 3 , 3 , 0 } , 24 } iex&gt; sibling ( % Pfx { bits : &lt;&lt; 10 , 11 &gt;&gt; , maxlen : 32 } , 1 ) % Pfx { bits : &lt;&lt; 10 , 12 &gt;&gt; , maxlen : 32 } iex&gt; sibling ( % Pfx { bits : &lt;&lt; 10 , 11 , 0 &gt;&gt; , maxlen : 32 } , 255 ) % Pfx { bits : &lt;&lt; 10 , 11 , 255 &gt;&gt; , maxlen : 32 } # wraps around iex&gt; sibling ( % Pfx { bits : &lt;&lt; 10 , 11 , 0 &gt;&gt; , maxlen : 32 } , 256 ) % Pfx { bits : &lt;&lt; 10 , 12 , 0 &gt;&gt; , maxlen : 32 } iex&gt; new ( &lt;&lt; 0 , 0 , 0 , 0 &gt;&gt; , 32 ) |&gt; sibling ( - 1 ) % Pfx { bits : &lt;&lt; 255 , 255 , 255 , 255 &gt;&gt; , maxlen : 32 } # zero bit-length stays zero bit-length iex&gt; sibling ( % Pfx { bits : &lt;&lt; &gt;&gt; , maxlen : 0 } , 1 ) % Pfx { bits : &lt;&lt; &gt;&gt; , maxlen : 0 }","ref":"Pfx.html#sibling/2","title":"Pfx.sibling/2","type":"function"},{"doc":"Returns the number of full addresses as represented by pfx . size(pfx) == 2^(pfx.maxlen - bit_size(pfx.bits)) Examples iex&gt; size ( &quot;1.1.1.0/23&quot; ) 512 iex&gt; size ( { 1 , 1 , 1 , 1 } ) 1 iex&gt; size ( { { 1 , 1 , 1 , 0 } , 16 } ) 65536 iex&gt; size ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } ) 256","ref":"Pfx.html#size/1","title":"Pfx.size/1","type":"function"},{"doc":"Returns true if given pfx is a teredo address, false otherwise IPv6 address within the teredo service prefix of 2000:0::/32 More details in rfc4380 . Examples iex&gt; teredo? ( &quot;2001:0000:4136:e378:8000:63bf:3fff:fdd2&quot; ) true iex&gt; teredo? ( &quot;1.1.1.1&quot; ) false iex&gt; teredo? ( 42 ) false","ref":"Pfx.html#teredo?/1","title":"Pfx.teredo?/1","type":"function"},{"doc":"Returns a map with the teredo address components of pfx or nil. Returns nil if pfx is not a teredo address. A teredo address consists of: the teredo service prefix of 2000:0::/32 IPv4 address of the teredo server flags (16 bits) that document type of address and NAT Port (16 bits), the obfuscated &quot;mapped UDP port&quot; at the client IPv4 address (obfucated) of the teredo client. More details in rfc4380 . Examples # example from https://en.wikipedia.org/wiki/Teredo_tunneling#IPv6_addressing iex&gt; teredo_decode ( &quot;2001:0000:4136:e378:8000:63bf:3fff:fdd2&quot; ) %{ server : &quot;65.54.227.120&quot; , client : &quot;192.0.2.45&quot; , port : 40000 , flags : { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } , prefix : &quot;2001:0000:4136:e378:8000:63bf:3fff:fdd2&quot; } iex&gt; teredo_decode ( { 0x2001 , 0 , 0x4136 , 0xe378 , 0x8000 , 0x63bf , 0x3fff , 0xfdd2 } ) %{ server : { 65 , 54 , 227 , 120 } , client : { 192 , 0 , 2 , 45 } , port : 40000 , flags : { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } , prefix : { 0x2001 , 0x0 , 0x4136 , 0xe378 , 0x8000 , 0x63bf , 0x3fff , 0xfdd2 } } iex&gt; teredo_decode ( &quot;1.1.1.1&quot; ) nil","ref":"Pfx.html#teredo_decode/1","title":"Pfx.teredo_decode/1","type":"function"},{"doc":"Encodes given server , client , port and flags as an IPv6 teredo address. The client and server must be full IPv4 adresses, while both port and flags are interpreted as 16-bit unsigned integers. The result mirrors the representation format of the client argument. Examples iex&gt; flags = { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } iex&gt; teredo_encode ( &quot;192.0.2.45&quot; , &quot;65.54.227.120&quot; , 40000 , flags ) &quot;2001:0:4136:e378:8000:63bf:3fff:fdd2&quot; iex&gt; flags = { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } iex&gt; teredo_encode ( { 192 , 0 , 2 , 45 } , &quot;65.54.227.120&quot; , 40000 , flags ) { 0x2001 , 0 , 0x4136 , 0xe378 , 0x8000 , 0x63bf , 0x3fff , 0xfdd2 } iex&gt; flags = { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } iex&gt; teredo_encode ( { { 192 , 0 , 2 , 45 } , 32 } , &quot;65.54.227.120&quot; , 40000 , flags ) { { 0x2001 , 0 , 0x4136 , 0xe378 , 0x8000 , 0x63bf , 0x3fff , 0xfdd2 } , 128 } iex&gt; flags = { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } iex&gt; teredo_encode ( % Pfx { bits : &lt;&lt; 192 , 0 , 2 , 45 &gt;&gt; , maxlen : 32 } , &quot;65.54.227.120&quot; , 40000 , flags ) % Pfx { bits : &lt;&lt; 0x2001 :: 16 , 0 :: 16 , 0x4136 :: 16 , 0xe378 :: 16 , 0x8000 :: 16 , 0x63bf :: 16 , 0x3fff :: 16 , 0xfdd2 :: 16 &gt;&gt; , maxlen : 128 }","ref":"Pfx.html#teredo_encode/4","title":"Pfx.teredo_encode/4","type":"function"},{"doc":"Returns a prefix in the form of a tuple, with or without mask. The options include: :mask whether to apply and include the mask (i.e. prefix length) :width how many bits in an address part (default depends) The :width defaults to 8 , except when maxlen is 128, in which case it is 16 . These defaults are a good fit for IPv4, IPv6, EUI48 and EUI64 prefixes and produces a tuple representation that can easily be converted back into a Pfx.t/0 struct if the need arises. Examples iex&gt; to_tuple ( &quot;1.1.1.0/24&quot; ) { { 1 , 1 , 1 , 0 } , 24 } # mask is applied by default iex&gt; to_tuple ( &quot;1.1.1.12/24&quot; ) { { 1 , 1 , 1 , 0 } , 24 } # unless :mask option is false iex&gt; to_tuple ( &quot;1.1.1.12/24&quot; , mask : false ) { 1 , 1 , 1 , 12 } # converts other formats of IPv4/6 and EUI48/64 as well iex&gt; to_tuple ( { 1 , 1 , 1 , 12 } ) { { 1 , 1 , 1 , 12 } , 32 } iex&gt; to_tuple ( % Pfx { bits : &lt;&lt; 1 , 1 , 1 , 12 &gt;&gt; , maxlen : 32 } ) { { 1 , 1 , 1 , 12 } , 32 } iex&gt; to_tuple ( &quot;acdc:1976::/32&quot; ) { { 0xacdc , 0x1976 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 } # convert back iex&gt; to_tuple ( &quot;acdc:1976::/32&quot; ) ...&gt; |&gt; new ( ) % Pfx { bits : &lt;&lt; 0xacdc :: size ( 16 ) , 0x1976 :: size ( 16 ) &gt;&gt; , maxlen : 128 } For other types of prefixes, use the :width option to override how many bits go into an address part. If the prefix has a maxlen that is not the number of address digits * their width, maxlen will need to be known when converting the tuple representation back into a Pfx.t/0 struct (since it cannot be deduced). # maxlen is 30 bits, not 8*4 iex&gt; to_tuple ( % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x56 &gt;&gt; , maxlen : 30 } , width : 4 ) { { 1 , 2 , 3 , 4 , 5 , 6 , 0 , 0 } , 24 } # conversion back into a Pfx struct requires `width` and `maxlen` to be known iex&gt; pfx = % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x56 &gt;&gt; , maxlen : 30 } iex&gt; { parts , pfxlen } = to_tuple ( pfx , width : 4 ) iex&gt; bits = for x &lt;- Tuple . to_list ( parts ) , into : &lt;&lt; &gt;&gt; , do : &lt;&lt; x :: size ( 4 ) &gt;&gt; iex&gt; new ( bits , 30 ) ...&gt; |&gt; keep ( pfxlen ) % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x56 &gt;&gt; , maxlen : 30 } # or less convoluted iex&gt; pfx = % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x56 &gt;&gt; , maxlen : 30 } iex&gt; to_tuple ( pfx , width : 4 ) ...&gt; |&gt; undigits ( 4 ) ...&gt; |&gt; new ( 30 ) % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x56 &gt;&gt; , maxlen : 30 }","ref":"Pfx.html#to_tuple/2","title":"Pfx.to_tuple/2","type":"function"},{"doc":"Trims all trailing 0 's from given prefix . Examples iex&gt; trim ( &quot;255.255.255.0&quot; ) &quot;255.255.255.0/24&quot; # perhaps more visible like thi iex&gt; trim ( &quot;255.255.255.0&quot; ) ...&gt; |&gt; new ( ) % Pfx { bits : &lt;&lt; 255 , 255 , 255 &gt;&gt; , maxlen : 32 } iex&gt; trim ( &quot;10.10.128.0/30&quot; ) &quot;10.10.128.0/17&quot; iex&gt; trim ( { 255 , 255 , 255 , 0 } ) { 255 , 255 , 255 , 0 } iex&gt; trim ( { { 255 , 255 , 255 , 0 } , 32 } ) { { 255 , 255 , 255 , 0 } , 24 } iex&gt; trim ( &quot;acdc:1976::ff00/128&quot; ) &quot;acdc:1976:0:0:0:0:0:ff00/120&quot;","ref":"Pfx.html#trim/1","title":"Pfx.trim/1","type":"function"},{"doc":"Returns the prefix type, one of :ip4 , :ip6 , :eui48 , eui64 or simply its maxlen property. Examples iex&gt; type ( &quot;1.2.3.4&quot; ) :ip4 iex&gt; type ( &quot;1.2.3.0/24&quot; ) :ip4 iex&gt; type ( { 1 , 2 , 3 , 4 } ) :ip4 iex&gt; type ( { { 1 , 2 , 3 , 4 } , 24 } ) :ip4 iex&gt; type ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } ) :ip4 iex&gt; type ( &quot;acdc:1976::1&quot; ) :ip6 iex&gt; type ( { 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 } ) :ip6 iex&gt; type ( { { 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 } , 64 } ) :ip6 iex&gt; type ( % Pfx { bits : &lt;&lt; &gt;&gt; , maxlen : 128 } ) :ip6 iex&gt; type ( &quot;aa-bb-cc-dd-ee-ff&quot; ) :eui48 iex&gt; type ( % Pfx { bits : &lt;&lt; 0xaa , 0xbb &gt;&gt; , maxlen : 48 } ) :eui48 iex&gt; type ( &quot;aa-bb-cc-ee-ff-00-00-00&quot; ) :eui64 iex&gt; type ( % Pfx { bits : &lt;&lt; 0xaa , 0xbb , 0xcc &gt;&gt; , maxlen : 64 } ) :eui64 iex&gt; type ( % Pfx { bits : &lt;&lt; 1 , 2 &gt;&gt; , maxlen : 256 } ) 256","ref":"Pfx.html#type/1","title":"Pfx.type/1","type":"function"},{"doc":"Returns the prefix represented by the digits , actual length and a given field width . The pfx.bits are formed by first concatenating the digits expressed as bitstrings of width -bits wide and then truncating to length bits. The pfx.maxlen is inferred as tuple_size(digits) * width . Note: if a digit does not fit in width -bits, only the width -least significant bits are preserved, which may yield surprising results. Examples # truncated to the first 24 bits and maxlen is 32 (4*8) iex&gt; undigits ( { { 10 , 11 , 12 , 0 } , 24 } , 8 ) % Pfx { bits : &lt;&lt; 10 , 11 , 12 &gt;&gt; , maxlen : 32 } iex&gt; undigits ( { { - 1 , - 1 , 0 , 0 } , 32 } , 8 ) |&gt; format ( ) &quot;255.255.0.0&quot; # bits are truncated to empty bitstring (`length` is 0) iex&gt; undigits ( { { 1 , 2 , 3 , 4 } , 0 } , 8 ) % Pfx { bits : &lt;&lt; &gt;&gt; , maxlen : 32 } # 32 4-bit wide numbers turn into an IPv6 prefix, truncated to 32 bits # and maxlen is set to 32 * 4 = 128 iex&gt; undigits ( { { 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } , 32 } , 4 ) % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x56 , 0x78 &gt;&gt; , maxlen : 128 }","ref":"Pfx.html#undigits/2","title":"Pfx.undigits/2","type":"function"},{"doc":"Returns true if pfx is designated as &quot;private-use&quot;. This includes the rfc1918 prefixes: 10.0.0.0/8 , 172.16.0.0/12 , and 192.168.0.0/16 . And the rfc4193 prefix fc00::/7 . Examples iex&gt; unique_local? ( &quot;172.31.255.255&quot; ) true iex&gt; unique_local? ( &quot;10.10.10.10&quot; ) true iex&gt; unique_local? ( &quot;fc00:acdc::&quot; ) true iex&gt; unique_local? ( &quot;172.32.0.0&quot; ) false iex&gt; unique_local? ( &quot;10.255.255.255&quot; ) true iex&gt; unique_local? ( { { 172 , 31 , 255 , 255 } , 32 } ) true iex&gt; unique_local? ( { 172 , 31 , 255 , 255 } ) true iex&gt; unique_local? ( % Pfx { bits : &lt;&lt; 172 , 31 , 255 , 255 &gt;&gt; , maxlen : 32 } ) true # bad prefix iex&gt; unique_local? ( &quot;10.255.255.256&quot; ) false","ref":"Pfx.html#unique_local?/1","title":"Pfx.unique_local?/1","type":"function"},{"doc":"Returns a boolean indicating whether pfx is a valid prefix/0 or not. Examples iex&gt; valid? ( &quot;1.2.3.4&quot; ) true iex&gt; valid? ( &quot;1.2.3.4/8&quot; ) true iex&gt; valid? ( { 1 , 2 , 3 , 4 } ) true iex&gt; valid? ( { { 1 , 2 , 3 , 4 } , 24 } ) true iex&gt; valid? ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } ) true # bits exceed maxlen iex&gt; valid? ( % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 16 } ) false","ref":"Pfx.html#valid?/1","title":"Pfx.valid?/1","type":"function"},{"doc":"An :inet IPv4 or IPv6 address (tuple)","ref":"Pfx.html#t:ip_address/0","title":"Pfx.ip_address/0","type":"type"},{"doc":"An IPv4 prefix ({ t:inet.ip4_address/0 , 0..32}) or an IPv6 prefix ({ t:inet.ip6_address/0 , 0..128}).","ref":"Pfx.html#t:ip_prefix/0","title":"Pfx.ip_prefix/0","type":"type"},{"doc":"A prefix expressed as either a Pfx.t/0 struct, an IP address-tuple, an address,length-tuple or a CIDR string.","ref":"Pfx.html#t:prefix/0","title":"Pfx.prefix/0","type":"type"},{"doc":"A prefix struct with fields: bits and maxlen .","ref":"Pfx.html#t:t/0","title":"Pfx.t/0","type":"type"},{"doc":"Pfx Functions to make working with prefixes easier, especially IP prefixes (IPv4 and IPv6). Pfx defines a prefix as a struct with a number of bits and a maximum maxlen length. Hence a Pfx struct represents some domain-specific value, like an IPv4/6 address or network, a MAC address, a MAC OUI range or something completely different. A Pfx struct can be created with Pfx.new/2 using: a bitstring/0 and a non_neg_integer/0 for the maximum length, a Pfx.t/0 and a non_neg_integer/0 for a (new) maximum length. This allows for the creation of any sort of prefix. Use Pfx.new/1 to create a prefix struct from: a Pfx.ip_address/0 , a Pfx.ip_prefix/0 , or a binary/0 , a string in IPv4 CIDR, IPv6, EUI-48 or EUI-64 format This creates an IPv4, IPv6, EUI-48 or an EUI-64 prefix. Other means to create prefixes include Pfx.from_mac/1 and Pfx.from_hex/1 . Several functions, like Pfx.unique_local?/1 are more IP oriented, and are included along with the more generic Pfx functions (like Pfx.cut/3 ) in order to have one module to rule them all. Functions generally accept either a Pfx.t/0 , a Pfx.ip_address/0 , a Pfx.ip_prefix/0 or a binary and yield their result in the same fashion: iex&gt; hosts ( &quot;10.10.10.0/30&quot; ) [ &quot;10.10.10.0&quot; , &quot;10.10.10.1&quot; , &quot;10.10.10.2&quot; , &quot;10.10.10.3&quot; ] iex&gt; hosts ( { { 10 , 10 , 10 , 0 } , 30 } ) [ { { 10 , 10 , 10 , 0 } , 32 } , { { 10 , 10 , 10 , 1 } , 32 } , { { 10 , 10 , 10 , 2 } , 32 } , { { 10 , 10 , 10 , 3 } , 32 } ] iex&gt; hosts ( % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 :: 6 &gt;&gt; , maxlen : 32 } ) [ % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 0 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 10 , 10 , 10 , 3 &gt;&gt; , maxlen : 32 } , ] # adopt representation of first argument iex&gt; band ( { 10 , 10 , 10 , 1 } , &quot;255.255.255.0&quot; ) { 10 , 10 , 10 , 0 } iex&gt; multicast? ( &quot;ff00::1&quot; ) true # MAC OUI prefix iex&gt; keep ( &quot;aa:bb:cc:dd:ee:ff&quot; , 24 ) &quot;AA-BB-CC-00-00-00/24&quot; # get IPv4 from a IPv4 compatible IPv6 address iex&gt; cut ( &quot;::1.2.3.4&quot; , - 1 , - 32 ) &quot;1.2.3.4&quot; # partition a range of address space iex&gt; partition_range ( &quot;10.10.10.0&quot; , &quot;10.10.10.33&quot; ) [ &quot;10.10.10.0/27&quot; , &quot;10.10.10.32/31&quot; ]","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"The Pfx.new/2 function will silently clip the provided bits -string to maxlen -bits when needed, since a Pfx struct named pfx is valid, iff: bit_size(pfx.bits) &lt;= pfx.maxlen , and where pfx.maxlen is a non_neg_integer/0 Keep that in mind when instantiating directly or updating a Pfx , otherwise functions will choke on it. Same goes for Pfx.ip_address/0 representations, which must be a valid :inet.ip_address() , representing either an IPv4 or IPv6 address through a tuple of four 8 -bit wide numbers or eight 16 -bit wide numbers. If used as the first element in a Pfx.ip_prefix/0 tuple, the second element is interpreted as the mask, used to clip the bitstring when creating the Pfx struct. For example: {{1, 1, 1, 0}, 24} is the same as 1.1.1.0/24 . IPv4 masks must be in range 0..32 and IPv6 masks in range 0..128 . The resulting Pfx will have its maxlen set to 32 for IPv4 tuples and 128 for IPv6 tuples. Last but not least, binaries are interpreted as either an IPv4 in CIDR-notation, an IPv6 address/prefix, an EUI-48 or EUI-64 formatted string. # IPv4 iex&gt; new ( &quot;1.2.3.4&quot; ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } iex&gt; new ( { 1 , 2 , 3 , 4 } ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , maxlen : 32 } iex&gt; new ( &quot;1.2.3.0/24&quot; ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } iex&gt; new ( { { 1 , 2 , 3 , 0 } , 24 } ) % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } # IPv6 iex&gt; new ( &quot;acdc:1975::&quot; ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1975 :: 16 , 0 :: 96 &gt;&gt; , maxlen : 128 } iex&gt; new ( { 44252 , 6517 , 0 , 0 , 0 , 0 , 0 , 0 } ) % Pfx { bits : &lt;&lt; 0xACDC :: 16 , 0x1975 :: 16 , 0 :: 96 &gt;&gt; , maxlen : 128 } # EUI-48 iex&gt; new ( &quot;00-88-88-88-88-88&quot; ) % Pfx { bits : &lt;&lt; 0 , 0x88 , 0x88 , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 48 } iex&gt; new ( &quot;0088.8888.8888&quot; ) % Pfx { bits : &lt;&lt; 0 , 0x88 , 0x88 , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 48 } # EUI-64 iex&gt; new ( &quot;02-88-88-FF-FE-88-88-88&quot; ) % Pfx { bits : &lt;&lt; 0x02 , 0x88 , 0x88 , 0xFF , 0xFE , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; new ( &quot;0288.88FF.FE88.8888&quot; ) % Pfx { bits : &lt;&lt; 0x02 , 0x88 , 0x88 , 0xFF , 0xFE , 0x88 , 0x88 , 0x88 &gt;&gt; , maxlen : 64 }","ref":"readme.html#validity","title":"Overview - Validity","type":"extras"},{"doc":"Pfx.new/1 accepts CIDR-strings which are ultimately processed using erlang's :inet.parse_address which, at the time of writing, still honors the ancient linux tradition of injecting zero's (rather than appending them) when presented with less than four IPv4 digits in a CIDR string. # &quot;d&quot; -&gt; &quot;0.0.0.d&quot; iex&gt; new ( &quot;10&quot; ) |&gt; format ( ) &quot;0.0.0.10&quot; iex&gt; new ( &quot;10/8&quot; ) |&gt; format ( ) &quot;0.0.0.0/8&quot; # &quot;d1.d2&quot; -&gt; &quot;d1.0.0.d2&quot; iex&gt; new ( &quot;10.10&quot; ) |&gt; format ( ) &quot;10.0.0.10&quot; iex&gt; new ( &quot;10.10/16&quot; ) |&gt; format ( ) &quot;10.0.0.0/16&quot; # &quot;d1.d2.d3&quot; -&gt; &quot;d1.d2.0.d3&quot; iex&gt; new ( &quot;10.10.10&quot; ) |&gt; format ( ) &quot;10.10.0.10&quot; iex&gt; new ( &quot;10.10.10/24&quot; ) |&gt; format ( ) &quot;10.10.0.0/24&quot; Bottom line: never go short, you may be unpleasantly surprised.","ref":"readme.html#ancient-tradition","title":"Overview - Ancient tradition","type":"extras"},{"doc":"Since a string is first parsed as an IP prefix, EUI-64's like &quot;11:22:33:44:55:66:77:88&quot; will come out as an IPv6 prefix with their maxlen property set to 128 . So, when parsing EUI's that might use ':'-s as punctuation, use Pfx.from_mac/1 , which also supports the tuple formats. Like Pfx.new/1 , this function always returns a Pfx.t/0 -struct. # new/1 parses EUI-64&#39;s like these correctly: iex&gt; new ( &quot;1122.3344.5566.7788&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } iex&gt; new ( &quot;11-22-33-44-55-66-77-88&quot; ) % Pfx { bits : &lt;&lt; 0x11 , 0x22 , 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 &gt;&gt; , maxlen : 64 } # but new/1 turns this valid EUI-64 into IPv6 due to &#39;:&#39;-punctuation used: iex&gt; new ( &quot;01:02:03:04:05:06:07:08&quot; ) % Pfx { bits : &lt;&lt; 0x1 :: 16 , 0x2 :: 16 , 0x3 :: 16 , 0x4 :: 16 , 0x5 :: 16 , 0x6 :: 16 , 0x7 :: 16 , 0x8 :: 16 &gt;&gt; , maxlen : 128 } # in this case, use from_mac/1 iex&gt; from_mac ( &quot;01:02:03:04:05:06:07:08&quot; ) % Pfx { bits : &lt;&lt; 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 &gt;&gt; , maxlen : 64 } # and supports digit-styled EUI&#39;s iex&gt; from_mac ( { 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 } ) % Pfx { bits : &lt;&lt; 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 &gt;&gt; , maxlen : 64 } # from {{digits}, len}, keeping first 3 bytes iex&gt; from_mac ( { { 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 0x7 , 0x8 } , 24 } ) % Pfx { bits : &lt;&lt; 0x1 , 0x2 , 0x3 &gt;&gt; , maxlen : 64 }","ref":"readme.html#eui-64-s","title":"Overview - EUI-64's","type":"extras"},{"doc":"A Pfx.t/0 implements the Enumerable protocol: iex&gt; for ip &lt;- % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 0 :: 6 &gt;&gt; , maxlen : 32 } , do : ip [ % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 0 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 3 &gt;&gt; , maxlen : 32 } , ]","ref":"readme.html#enumeration","title":"Overview - Enumeration","type":"extras"},{"doc":"Pfx.t/0 implements the String.Chars protocol with some defaults for prefixes that formats prefixes with: maxlen: 32 as an IPv4 CIDR string, maxlen: 48 as a EUI-48 address string and maxlen: 64 as a EUI-64 address string maxlen: 128 as an IPv6 string Other maxlen 's will simply come out as a series of 8-bit numbers joined by &quot;.&quot; followed by /num_of_bits . The latter is omitted if equal to pfx.bits length. If other formatting is required, use the Pfx.format/2 function, which takes some options that help shape the string representation for a Pfx struct. iex&gt; &quot; \#{ % Pfx { bits : &lt;&lt; 10 , 11 , 12 &gt;&gt; , maxlen : 32 } } &quot; &quot;10.11.12.0/24&quot; iex&gt; &quot; \#{ new ( &lt;&lt; 44252 :: 16 , 6518 :: 16 &gt;&gt; , 128 ) } &quot; &quot;acdc:1976:0:0:0:0:0:0/32&quot; iex&gt; &quot; \#{ % Pfx { bits : &lt;&lt; 0xA1 , 0xB2 , 0xC3 , 0xD4 , 0xE5 , 0xF6 &gt;&gt; , maxlen : 48 } } &quot; &quot;A1-B2-C3-D4-E5-F6&quot; iex&gt; &quot; \#{ new ( &lt;&lt; 1 , 2 , 3 , 4 , 5 &gt;&gt; , 64 ) } &quot; &quot;01-02-03-04-05-00-00-00/40&quot; # the enumeration example earlier, could also read: iex&gt; for ip &lt;- new ( &quot;1.2.3.0/30&quot; ) , do : &quot; \#{ ip } &quot; [ &quot;1.2.3.0&quot; , &quot;1.2.3.1&quot; , &quot;1.2.3.2&quot; , &quot;1.2.3.3&quot; ] # or iex&gt; for ip &lt;- new ( &quot;1.2.3.0/30&quot; ) , do : digits ( ip , 8 ) |&gt; elem ( 0 ) [ { 1 , 2 , 3 , 0 } , { 1 , 2 , 3 , 1 } , { 1 , 2 , 3 , 2 } , { 1 , 2 , 3 , 3 } ]","ref":"readme.html#string-chars","title":"Overview - String.Chars","type":"extras"},{"doc":"A lot of Pfx -functions convert the Pfx.bits bitstring to an integer using Pfx.cast/1 , before performing some, often Bitwise -related, calculation on them. Luckily Elixir can handle pretty large numbers which seem mostly limited by the available system memory. Other functions, like Pfx.digits/2 return a tuple with numbers and are so limited by the maximum number of elements in a tuple (~16M+). So if you're taking this somewhere far, far away, heed these limitations before take off. Also, everything is done in Elixir with no extra, external dependencies. Usually fast enough, but if you really feel the need for speed, you might want to look elsewhere. Anyway, enough downplay, here are some more examples.","ref":"readme.html#limitations","title":"Overview - Limitations","type":"extras"},{"doc":"# IANA&#39;s OUI range 00-00-5e-xx-xx-xx iex&gt; new ( &quot;00-00-5e-00-00-00/24&quot; ) % Pfx { bits : &lt;&lt; 0 , 0 , 94 &gt;&gt; , maxlen : 48 } # IANA&#39;s VRRP MAC address range 00-00-5e-00-01-{VRID} iex&gt; vrrp_mac_range = new ( &quot;00-00-5e-00-01-00/40&quot; ) % Pfx { bits : &lt;&lt; 0 , 0 , 94 , 0 , 1 &gt;&gt; , maxlen : 48 } iex&gt; iex&gt; vrrp_mac = new ( &quot;00-00-5e-00-01-0f&quot; ) % Pfx { bits : &lt;&lt; 0 , 0 , 94 , 0 , 1 , 15 &gt;&gt; , maxlen : 48 } iex&gt; iex&gt; member? ( vrrp_mac , vrrp_mac_range ) true iex&gt; cut ( vrrp_mac , - 1 , - 8 ) |&gt; cast ( ) 15 iex&gt; new ( &quot;10.10.10.0/24&quot; ) % Pfx { bits : &lt;&lt; 10 , 10 , 10 &gt;&gt; , maxlen : 32 } iex&gt; mask ( &quot;10.10.10.0/25&quot; ) &quot;255.255.255.128&quot; iex&gt; inv_mask ( &quot;10.10.10.0/25&quot; ) &quot;0.0.0.127&quot; iex&gt; dns_ptr ( &quot;acdc:1975::b1ba:2021&quot; ) &quot;1.2.0.2.a.b.1.b.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.5.7.9.1.c.d.c.a.ip6.arpa&quot; iex&gt; teredo_decode ( &quot;2001:0000:4136:e378:8000:63bf:3fff:fdd2&quot; ) %{ server : &quot;65.54.227.120&quot; , client : &quot;192.0.2.45&quot; , port : 40000 , flags : { 1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 } , prefix : &quot;2001:0000:4136:e378:8000:63bf:3fff:fdd2&quot; } iex&gt; eui64_encode ( &quot;0288.8888.8888&quot; ) &quot;00-88-88-FF-FE-88-88-88&quot; Some functions have IP related names (like Pfx.teredo_decode/1 , but most of the times functions have generic names, since they apply to all sorts of prefixes, e.g. iex&gt; partition ( &quot;10.10.10.0/24&quot; , 26 ) [ &quot;10.10.10.0/26&quot; , &quot;10.10.10.64/26&quot; , &quot;10.10.10.128/26&quot; , &quot;10.10.10.192/26&quot; ] iex&gt; new ( &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , 32 ) ...&gt; |&gt; format ( width : 1 , unit : 8 ) &quot;00000001.00000010.00000011.00000100&quot; iex&gt; from_hex ( &quot;123456789abcdef&quot; ) ...&gt; |&gt; keep ( 20 ) % Pfx { bits : &lt;&lt; 0x12 , 0x34 , 0x5 :: 4 &gt;&gt; , maxlen : 60 } iex&gt; brot ( &quot;1.2.3.4&quot; , 8 ) &quot;4.1.2.3&quot;","ref":"readme.html#examples","title":"Overview - Examples","type":"extras"},{"doc":"Pfx can be installed by adding pfx to your list of dependencies in mix.exs : def deps do [ { :pfx , &quot;~&gt; 0.10.1&quot; } ] end","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"Copyright (c) 2021 hertogp The source code is licensed under the MIT License .","ref":"readme.html#copyright-and-license","title":"Overview - Copyright and License","type":"extras"},{"doc":"MIT License Copyright (c) 2021 hertogp Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"CHANGELOG All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"fixed Pfx.member/3 , check width argument for validity and raise ArgumentError if invalid fixed the Enumerable.Pfx.slice/1 implementation, so Enum.take &amp; friends work properly","ref":"changelog.html#0-10-1-2022-01-09","title":"CHANGELOG - [0.10.1] - 2022-01-09","type":"extras"},{"doc":"added Pfx.partition_range/2 which returns a list of prefixes, where range is given by: a start prefix and a stop prefix, or a start prefix and a number of hosts Pfx.to_tuple/2 to convert (with options) a prefix into an address-tuple option :mask , default true, applies the mask and yields an address,length-tuple option :width default 8 (or 16 in case maxlen is 128) the width option allows for converting prefixes other than ipv4/6, eui48/64 as well","ref":"changelog.html#v0-10-0-2021-12-29","title":"CHANGELOG - [v0.10.0] - 2021-12-29","type":"extras"},{"doc":"added Pfx.address/1 , returns the address portion of given prefix without applying a mask. Pfx.type/1 which returns either :ip4, :ip6, :eui48, :eui64 or the prefix.maxlen property fixed typespec for Pfx.new now includes Pfx.t as argument as well typespec for some others too","ref":"changelog.html#v0-9-0-2021-12-03","title":"CHANGELOG - [v0.9.0] - 2021-12-03","type":"extras"},{"doc":"added tests for Pfx.pfxlen/1 added Pfx.parse/1 added Pfx.parse/2 changed type spec for t:Pfx.prefix/t now uses t instead of Pfx.t()","ref":"changelog.html#v0-8-0-2021-11-15","title":"CHANGELOG - [v0.8.0] - 2021-11-15","type":"extras"},{"doc":"added Pfx.pfxlen/1 returns the number of bits in the prefix changed Pfx.mask/2 now takes 2 options (inv_mask and trim) and trims result by default","ref":"changelog.html#v0-7-0-2021-07-27","title":"CHANGELOG - [v0.7.0] - 2021-07-27","type":"extras"},{"doc":"added Pfx.trim/1 trims all trailing zero's from given prefix Pfx.mask/2 applies mask to prefix (changing its prefix.bits string)","ref":"changelog.html#v0-6-0-2021-07-22","title":"CHANGELOG - [v0.6.0] - 2021-07-22","type":"extras"},{"doc":"changed &quot;Pfx.multicast/1&quot; renamed to Pfx.multicast_decode/1 and updated decoding Pfx.eui64_encode/1 now also accepts EUI-64 (only needs to flip 7th bit) added added Pfx.first/1 as a more generic version of Pfx.network/1 added Pfx.last/1 as a more generic version of Pfx.broadcast/1 added Pfx.from_hex/1 to create a Pfx.t/0 from any hex string","ref":"changelog.html#v0-5-0-2021-07-17","title":"CHANGELOG - [v0.5.0] - 2021-07-17","type":"extras"},{"doc":"added Pfx.eui64_encode/1 , which creates a modified EUI-64 from a EUI-48 address Pfx.eui64_decode/1 , which reverses a modified EUI-64 to an EUI-48 address Pfx.flip/2 , which flips a single bit in bitstring Pfx.insert/3 , which inserts some bits into bitstring Pfx.remove/3 , which removes some bits from bitstring changed Pfx.new/1 and Pfx.from_mac/1 also parse EUI-64 in Cisco's dot format Pfx.teredo_decode/1 mirrors the representation for server/client to its pfx argument functions raise their own exceptions when marshalling (instead of leaving that up to Pfx.new/1 )","ref":"changelog.html#v0-4-0-2021-07-10","title":"CHANGELOG - [v0.4.0] - 2021-07-10","type":"extras"},{"doc":"added Pfx.keep/2 to keep some msb bits String.Chars for MAC addresses with maxlen 64 (w/ hyphens) Pfx.from_mac/1 to support EUI-64 with &quot;:&quot;'s changed Pfx.new/1 now parses EUI-48 as well Pfx.new/1 now parses EUI-64 as well, unless using ':' for punctuation String.Chars for MAC addresses with maxlen 48 uses hyphens String.Chars no longer accepts invalid Pfx.t/0 structs Pfx.format/2 without opts , uses same defaults as String.Chars","ref":"changelog.html#v0-3-0-2021-07-08","title":"CHANGELOG - [v0.3.0] - 2021-07-08","type":"extras"},{"doc":"added Pfx.marshall/2 is now a public function changed renamed Pfx.teredo to Pfx.teredo_decode/1","ref":"changelog.html#v0-2-1-2021-07-04","title":"CHANGELOG - [v0.2.1] - 2021-07-04","type":"extras"},{"doc":"added Pfx.teredo_encode/4 Pfx.drop/2 , which drops some lsb bits improved documentation","ref":"changelog.html#v0-2-0-2021-07-03","title":"CHANGELOG - [v0.2.0] - 2021-07-03","type":"extras"},{"doc":"changed Pfx.band/2 , result has same number of bits as its first argument Pfx.bor/2 , result has same number of bits as its first argument Pfx.bxor/2 , result has same number of bits as its first argument IPv6 addresses in lowercase fixed Pfx.brot/2 , won't choke on rotating empty pfx.bits Pfx.contrast/2 , 1.2.3.0/24 is really to the left of 1.2.4.0/24","ref":"changelog.html#v0-1-1-2021-07-03","title":"CHANGELOG - [v0.1.1] - 2021-07-03","type":"extras"},{"doc":"initial version","ref":"changelog.html#v0-1-0-2021-06-28","title":"CHANGELOG - [v0.1.0] - 2021-06-28","type":"extras"}]